#!/usr/bin/env python3
# -*- coding: utf-8 -*-
"""
Created on Wed May 30 07:34:40 2018

@author: prmiles
"""
from __future__ import print_function

from pymcmcstat.utilities import progressbar as pbar
import unittest
from mock import patch
import io
import sys

# --------------------------
class InitializeProgress_Bar(unittest.TestCase):

    def test_PB_iteration_assignment(self):
        PB = pbar.progress_bar(iters = 100)
        self.assertEqual(PB.iterations, 100, msg = 'Iteration assignment should be 100')
        
    def test_PB_animation_interval(self):
        PB = pbar.progress_bar(iters = 100)
        self.assertEqual(PB.animation_interval, 0.5, msg = 'Animation interval should be 0.5')
        
# --------------------------
class BaseProgressBar(unittest.TestCase):

    def test_initialize_progress_bar(self):
        PB = pbar.ProgressBar(iterations=100)
        self.assertEqual(PB.iterations, 100, msg = 'Iterations = 100')
        self.assertEqual(PB.animation_interval, 0.5, msg = 'Animation interval = 0.5')
        self.assertEqual(PB.last, 0, msg = 'last = 0')
        
    def test_initialize_progress_bar_change_animation_interval(self):
        PB = pbar.ProgressBar(iterations=100, animation_interval=0.78)
        self.assertEqual(PB.iterations, 100, msg = 'Iterations = 100')
        self.assertEqual(PB.animation_interval, 0.78, msg = 'Animation interval = 0.5')
        self.assertEqual(PB.last, 0, msg = 'last = 0')
        
    def test_return_percentage(self):
        PB = pbar.ProgressBar(iterations=100)
        i = 37
        self.assertEqual(PB.percentage(i = i), 100 * i / float(100), msg = 'Percentage = 37%')
        
#    def test_update(self):
#        PB = pbar.progress_bar(iters=100)
#        i = 37
#        self.assertEqual(PB.update(i = i), 1, msg = 'None')
        
# --------------------------
class TextProgressBar(unittest.TestCase):
    
    def test_initialization_text(self):
        TPB = pbar.TextProgressBar(iterations = 100, printer=pbar.consoleprint)
        self.assertEqual(TPB.width, 40, msg = 'Width is 40')
        self.assertEqual(TPB.fill_char, '-', msg = '-')
        self.assertEqual(TPB.printer, pbar.consoleprint, msg = 'consoleprint')
        
    def test_initialization_text_with_ipythonprint(self):
        TPB = pbar.TextProgressBar(iterations = 100, printer=pbar.ipythonprint)
        self.assertEqual(TPB.width, 40, msg = 'Width is 40')
        self.assertEqual(TPB.fill_char, '-', msg = '-')
        self.assertEqual(TPB.printer, pbar.ipythonprint, msg = 'ipythonprint')
        
    def test_progbar(self):
        TPB = pbar.TextProgressBar(iterations = 100, printer=pbar.consoleprint)
        self.assertTrue(isinstance(TPB.progbar(i = 50, elapsed=50),str), msg = 'Expected string return')

# --------------------------
class IpythonProgressBarMock(unittest.TestCase):
    @patch('pymcmcstat.utilities.progressbar.run_from_ipython')
    def test_ipython_console(self, mock_simple_func):
        mock_simple_func.return_value = True
        PB = pbar.progress_bar(iters = 100)
        self.assertEqual(PB.printer, pbar.ipythonprint, msg = 'ipythonprint')

# --------------------------
class IpythonPrint(unittest.TestCase):
    
    def test_ipythonprint(self):
        capturedOutput = io.StringIO()                  # Create StringIO object
        sys.stdout = capturedOutput                     #  and redirect stdout.
        pbar.ipythonprint('test')                                     # Call function.
        sys.stdout = sys.__stdout__                     # Reset redirect.
        self.assertEqual(capturedOutput.getvalue(), '\r test', msg = 'Expected string')

# --------------------------
class ConsolePrint(unittest.TestCase):
    
    def test_standard_print(self):
        capturedOutput = io.StringIO()                  # Create StringIO object
        sys.stdout = capturedOutput                     #  and redirect stdout.
        pbar.consoleprint('test')                                     # Call function.
        sys.stdout = sys.__stdout__                     # Reset redirect.
        self.assertEqual(capturedOutput.getvalue(), 'test\n', msg = 'Expected string')
       
    @patch('pymcmcstat.utilities.progressbar.check_windows_platform')
    def test_windows_print(self, mock_simple_func):
        mock_simple_func.return_value = True
        capturedOutput = io.StringIO()                  # Create StringIO object
        sys.stdout = capturedOutput                     #  and redirect stdout.
        pbar.consoleprint('test')                                     # Call function.
        sys.stdout = sys.__stdout__                     # Reset redirect.
        self.assertEqual(capturedOutput.getvalue(), 'test \r', msg = 'Expected string')


#!/usr/bin/env python2
# -*- coding: utf-8 -*-
"""
Created on Wed Jan 17 16:21:48 2018

@author: prmiles
"""

# Import required packages
import numpy as np
import sys

class SumOfSquares:
    '''
    Sum-of-squares function evaluation.

    **Description:** Sum-of-squares (sos) class intended for used in MCMC simulator.  Each instance
    will contain the sos function.  If the user did not specify a sos-function,
    then the user supplied model function will be used in the default mcmc sos-function.

    Attributes:
        * :meth:`evaluate_sos_function`
        * :meth:`mcmc_sos_function`
    '''
    def __init__(self, model, data, parameters):
        # check if sos function and model function are defined
        if model.sos_function is None: #isempty(ssfun)
            if model.model_function is None: #isempty(modelfun)
                sys.exit('No ssfun or modelfun specified!')
            sos_style = 4
        else:
            sos_style = 1
        
        self.sos_function = model.sos_function
        self.sos_style = sos_style
        self.model_function = model.model_function
        self.parind = parameters._parind
        self.value = parameters._initial_value
        self.local = parameters._local
        self.data = data
        self.nbatch = model.nbatch
        
    def evaluate_sos_function(self, theta):
        '''
        Evaluate sum-of-squares function.

        Args:
            * **theta** (:class:`~numpy.ndarray`): Parameter values.

        Returns:
            * **ss** (:class:`~numpy.ndarray`): Sum-of-squares error(s)
        '''
        # evaluate sum-of-squares function
        self.value[self.parind] = theta
        if self.sos_style == 1:
            ss = self.sos_function(self.value, self.data)
        elif self.sos_style == 4:
            ss = self.mcmc_sos_function(self.value, self.data, self.nbatch, self.model_function)
        else:
            ss = self.sos_function(self.value, self.data, self.local)
        
        # make sure sos is a numpy array
        if not isinstance(ss, np.ndarray):
            ss = np.array([ss])
            
        return ss
     
    @classmethod
    def mcmc_sos_function(cls, theta, data, nbatch, model_function):
        '''
        Default sum-of-squares function.

        .. note::

            This method requires specifying a model function instead of a
            sum of squares function.  Not recommended for most applications.

        Basic formulation:

        .. math::

            SS_{q,i} = \sum [w_i(y^{data}_i-y^{model}_i)^2]

        where :math:`w_i` is the weight of a particular data set, and :math:`SS_{q,i}`
        is the sum-of-squares error for the `i`-th data set.

        Args:
            * **theta** (:class:`~numpy.ndarray`): Parameter values.

        Returns:
            * **ss** (:class:`~numpy.ndarray`): Sum-of-squares error(s)           
        '''
        # initialize
        ss = np.zeros(nbatch)

        for ibatch in range(nbatch):
                xdata = data.xdata[ibatch]
                ydata = data.ydata[ibatch]
                weight = data.weight[ibatch]
            
                # evaluate model
                ymodel = model_function(xdata, theta)
    
                # calculate sum-of-squares error
                ss[ibatch] += sum(weight*(ydata-ymodel)**2)
        
        return ss


#!/usr/bin/env python2
# -*- coding: utf-8 -*-
"""
Created on Mon Apr  2 08:42:56 2018

@author: prmiles
"""

import numpy as np
import json

class NumpyEncoder(json.JSONEncoder):
    '''
    Encoder used for storing numpy arrays in json files.
    '''
    def default(self, obj):
        if isinstance(obj, np.ndarray):
            return obj.tolist()
        return json.JSONEncoder.default(self, obj)


#!/usr/bin/env python3
# -*- coding: utf-8 -*-
"""
Created on Thu Apr 26 08:48:00 2018

@author: prmiles
"""

from pymcmcstat.samplers.DelayedRejection import update_set_based_on_acceptance, extract_state_elements
from pymcmcstat.samplers.DelayedRejection import log_posterior_ratio, nth_stage_log_proposal_ratio
from pymcmcstat.samplers.DelayedRejection import DelayedRejection
from pymcmcstat.structures.ParameterSet import ParameterSet
from pymcmcstat.procedures.SumOfSquares import SumOfSquares
from pymcmcstat.procedures.PriorFunction import PriorFunction
import test.general_functions as gf

import unittest
from mock import patch
import numpy as np

# --------------------------
class InitializeDRMetrics(unittest.TestCase):
    def test_dr_metrics(self):
        DR = DelayedRejection()
        __, options, parameters, data, covariance, rejected, chain, s2chain, sschain = gf.setup_mcmc_case_dr()
        DR._initialize_dr_metrics(options = options)
        self.assertTrue(np.array_equal(DR.iacce, np.zeros(options.ntry, dtype = int)), msg = 'Arrays should match')
        self.assertEqual(DR.dr_step_counter, 0, msg = 'Counter initialized to zero')
        
# --------------------------        
class InitializeNextMetropolisStep(unittest.TestCase):
    @patch('numpy.random.randn', return_value = np.array([0.2, 0.5]))
    def test_next_set(self, mock_1):
        DR = DelayedRejection()
        npar = 2
        old_theta = np.array([0.1, 0.2])
        RDR = np.array([[0.4, 0.2],[0, 0.3]])
        sigma2 = 0.24
        next_set = DR.initialize_next_metropolis_step(npar = npar, old_theta = old_theta, sigma2 = sigma2, RDR = RDR)
        self.assertEqual(next_set.sigma2, sigma2, msg = 'sigma2 should be 0.24')
        self.assertTrue(np.array_equal(next_set.theta, (old_theta + np.dot(np.array([0.2,0.5]),RDR)).reshape(npar)), msg = 'Arrays should match')
        
# -------------------------------------------
class UpdateSetBasedOnAcceptance(unittest.TestCase):
    def test_set_based_on_accept_false(self):
        next_set = ParameterSet(theta = np.random.random_sample(size = (2,1)), ss = 0.4)
        old_set = ParameterSet(theta = np.random.random_sample(size = (2,1)), ss = 0.6)
        out_set = update_set_based_on_acceptance(accept = False, old_set = old_set, next_set = next_set)
        self.assertEqual(out_set, old_set)
        
    def test_set_based_on_accept_true(self):
        next_set = ParameterSet(theta = np.random.random_sample(size = (2,1)), ss = 0.4)
        old_set = ParameterSet(theta = np.random.random_sample(size = (2,1)), ss = 0.6)
        out_set = update_set_based_on_acceptance(accept = True, old_set = old_set, next_set = next_set)
        self.assertEqual(out_set, next_set)
        
# -------------------------------------------
class ExtractStateElements(unittest.TestCase):
    def test_extract_state_elements_iq_eq_0(self):
        iq = 0
        trypath = []
        trypath.append(ParameterSet(theta = 0.1))
        trypath.append(ParameterSet(theta = 0.2))
        trypath.append(ParameterSet(theta = 0.3))
        trypath.append(ParameterSet(theta = 0.4))
        stage = len(trypath) - 2
        y1, y2, y3, y4 = extract_state_elements(iq = iq, stage = stage, trypath = trypath)
        self.assertEqual(y1, trypath[0].theta, msg = 'Expect [0]')
        self.assertEqual(y2, trypath[1].theta, msg = 'Expect [1]')
        self.assertEqual(y3, trypath[3].theta, msg = 'Expect [3]')
        self.assertEqual(y4, trypath[2].theta, msg = 'Expect [2]')
        
    def test_extract_state_elements_iq_eq_1(self):
        iq = 1
        trypath = []
        trypath.append(ParameterSet(theta = 0.1))
        trypath.append(ParameterSet(theta = 0.2))
        trypath.append(ParameterSet(theta = 0.3))
        trypath.append(ParameterSet(theta = 0.4))
        stage = len(trypath) - 2
        y1, y2, y3, y4 = extract_state_elements(iq = iq, stage = stage, trypath = trypath)
        self.assertEqual(y1, trypath[0].theta, msg = 'Expect [0]')
        self.assertEqual(y2, trypath[2].theta, msg = 'Expect [2]')
        self.assertEqual(y3, trypath[3].theta, msg = 'Expect [3]')
        self.assertEqual(y4, trypath[1].theta, msg = 'Expect [1]')
        
    def test_extract_state_elements_iq_eq_2(self):
        iq = 2
        trypath = []
        trypath.append(ParameterSet(theta = 0.1))
        trypath.append(ParameterSet(theta = 0.2))
        trypath.append(ParameterSet(theta = 0.3))
        trypath.append(ParameterSet(theta = 0.4))
        stage = len(trypath) - 2
        y1, y2, y3, y4 = extract_state_elements(iq = iq, stage = stage, trypath = trypath)
        self.assertEqual(y1, trypath[0].theta, msg = 'Expect [0]')
        self.assertEqual(y2, trypath[3].theta, msg = 'Expect [3]')
        self.assertEqual(y3, trypath[3].theta, msg = 'Expect [3]')
        self.assertEqual(y4, trypath[0].theta, msg = 'Expect [0]')
        
# -------------------------------------------
class LogPosteriorRatio(unittest.TestCase):
    def test_logposteriorratio(self):
        trypath = []
        trypath.append(ParameterSet(theta = 0.1, ss = np.array([10.2]), sigma2 = np.array([0.5]), prior = np.array([0.5])))
        trypath.append(ParameterSet(theta = 0.2, ss = np.array([8.2]), sigma2 = np.array([0.5]), prior = np.array([0.5])))
        zq = log_posterior_ratio(x1 = trypath[0], x2 = trypath[-1])
        self.assertTrue(isinstance(zq, float), msg = 'Expect float return')
        self.assertTrue(zq > 0., msg = 'Expect positve return')
        
    def test_logposteriorratio_with_2d(self):
        trypath = []
        trypath.append(ParameterSet(theta = 0.1, ss = np.array([10.2, 5.1]), sigma2 = np.array([0.5, 0.6]), prior = np.array([0.5, 0.75])))
        trypath.append(ParameterSet(theta = 0.2, ss = np.array([8.2, 7.5]), sigma2 = np.array([0.5, 1.2]), prior = np.array([0.5, 0.25])))
        zq = log_posterior_ratio(x1 = trypath[0], x2 = trypath[-1])
        self.assertTrue(isinstance(zq, float), msg = 'Expect float return')
        
# -------------------------------------------
class NthStateLogProposalRatio(unittest.TestCase):
    def test_nth_stage_logpropratio(self):
        iq = 0
        trypath = []
        trypath.append(ParameterSet(theta = 0.1, ss = np.array([10.2]), sigma2 = np.array([0.5]), prior = np.array([0.5])))
        trypath.append(ParameterSet(theta = 0.2, ss = np.array([8.2]), sigma2 = np.array([0.5]), prior = np.array([0.5])))
        zq = nth_stage_log_proposal_ratio(iq = iq, trypath = trypath, invR = None)
        self.assertTrue(np.array_equal(zq, np.zeros([1])), msg = 'Expect arrays to match')
        
    def test_nth_stage_logpropratio_invR(self):
        iq = 0
        invR = [np.array([[0.4, 0.1],[0., 0.2]])]
        trypath = []
        trypath.append(ParameterSet(theta = 0.1, ss = np.array([10.2]), sigma2 = np.array([0.5]), prior = np.array([0.5])))
        trypath.append(ParameterSet(theta = 0.2, ss = np.array([8.2]), sigma2 = np.array([0.5]), prior = np.array([0.5])))
        trypath.append(ParameterSet(theta = 0.2, ss = np.array([8.2]), sigma2 = np.array([0.5]), prior = np.array([0.5])))
        zq = nth_stage_log_proposal_ratio(iq = iq, trypath = trypath, invR = invR)
        self.assertTrue(isinstance(zq, float), msg = 'Expect float return')
        
# -------------------------------------------
class AlphaFunction(unittest.TestCase):
    def test_alphafun(self):
        invR = []
        invR.append(np.array([[0.4, 0.1],[0., 0.2]]))
        invR.append(np.array([[0.4, 0.1],[0., 0.2]])/4)
        invR.append(np.array([[0.4, 0.1],[0., 0.2]])/5)
        trypath = []
        trypath.append(ParameterSet(theta = 0.1, ss = np.array([10.2]), sigma2 = np.array([0.5]), prior = np.array([0.5])))
        trypath.append(ParameterSet(theta = 0.2, ss = np.array([8.2]), sigma2 = np.array([0.5]), prior = np.array([0.5])))
        trypath.append(ParameterSet(theta = 0.2, ss = np.array([8.2]), sigma2 = np.array([0.5]), prior = np.array([0.5])))
        trypath.append(ParameterSet(theta = 0.2, ss = np.array([8.2]), sigma2 = np.array([0.5]), prior = np.array([0.5])))
        __, options, __, __ = gf.setup_mcmc()
        DR = DelayedRejection()
        DR._initialize_dr_metrics(options = options)
        alpha = DR.alphafun(trypath = trypath, invR = invR)
        self.assertIsInstance(alpha, np.ndarray, msg = 'Expect numpy array return')
        self.assertEqual(alpha.size, 1, msg = 'Expect single element array')
        
# -------------------------------------------
class RunDelayedRejection(unittest.TestCase):
    @patch('pymcmcstat.samplers.DelayedRejection.acceptance_test', return_value = True)
    def test_run_dr(self, mock_accept):
        model, options, parameters, data, covariance, __, __, __, __ = gf.setup_mcmc_case_dr()
        RDR = covariance._RDR
        invR = covariance._invR
        old_set = ParameterSet(theta = 0.1, ss = np.array([10.2]), sigma2 = np.array([0.5]), prior = np.array([0.5]))
        new_set = ParameterSet(theta = 0.2, ss = np.array([8.2]), sigma2 = np.array([0.5]), prior = np.array([0.5]))
        priorobj = PriorFunction(priorfun = model.prior_function, mu = parameters._thetamu, sigma = parameters._thetasigma)
        sosobj = SumOfSquares(model, data, parameters)
        DR = DelayedRejection()
        DR._initialize_dr_metrics(options = options)
        accept, out_set, outbound = DR.run_delayed_rejection(old_set = old_set, new_set = new_set, RDR = RDR, ntry = 2, parameters = parameters, invR = invR, sosobj = sosobj, priorobj = priorobj)
        self.assertTrue(accept, msg = 'Expect return True')
        
    @patch('pymcmcstat.samplers.DelayedRejection.is_sample_outside_bounds', return_value = True)
    def test_run_dr_outside(self, mock_outside):
        model, options, parameters, data, covariance, __, __, __, __ = gf.setup_mcmc_case_dr()
        RDR = covariance._RDR
        invR = covariance._invR
        old_set = ParameterSet(theta = 0.1, ss = np.array([10.2]), sigma2 = np.array([0.5]), prior = np.array([0.5]))
        new_set = ParameterSet(theta = 0.2, ss = np.array([8.2]), sigma2 = np.array([0.5]), prior = np.array([0.5]))
        priorobj = PriorFunction(priorfun = model.prior_function, mu = parameters._thetamu, sigma = parameters._thetasigma)
        sosobj = SumOfSquares(model, data, parameters)
        DR = DelayedRejection()
        DR._initialize_dr_metrics(options = options)
        accept, out_set, outbound = DR.run_delayed_rejection(old_set = old_set, new_set = new_set, RDR = RDR, ntry = 2, parameters = parameters, invR = invR, sosobj = sosobj, priorobj = priorobj)
        self.assertFalse(accept, msg = 'Expect return False')
        self.assertTrue(outbound, msg = 'Expect return True')
        self.assertIsInstance(out_set, ParameterSet, msg = 'Expect structure')


#!/usr/bin/env python3
# -*- coding: utf-8 -*-
"""
Created on Wed Apr 11 11:14:18 2018

Description: This file contains a series of utilities designed to test the
features in the 'DataStructure.py" package of the pymcmcstat module.

@author: prmiles
"""
from pymcmcstat.settings import DataStructure
import unittest
import numpy as np

# --------------------------
# initialization
# --------------------------
class Data_Structure_Initialization(unittest.TestCase):
    def test_does_initialization_yield_xdata(self):
        data = DataStructure.DataStructure()
        self.assertTrue(hasattr(data,'xdata'))
    def test_does_initialization_yield_ydata(self):
        data = DataStructure.DataStructure()
        self.assertTrue(hasattr(data,'ydata'))
    def test_does_initialization_yield_n(self):
        data = DataStructure.DataStructure()
        self.assertTrue(hasattr(data,'n'))
    def test_does_initialization_yield_shape(self):
        data = DataStructure.DataStructure()
        self.assertTrue(hasattr(data,'shape'))
    def test_does_initialization_yield_weight(self):
        data = DataStructure.DataStructure()
        self.assertTrue(hasattr(data,'weight'))
    def test_does_initialization_yield_udobj(self):
        data = DataStructure.DataStructure()
        self.assertTrue(hasattr(data,'user_defined_object'))
        
# --------------------------
# adding data
# --------------------------
class Add_Data_Set(unittest.TestCase):
    
    def test_is_xdata_a_list(self):
        data = DataStructure.DataStructure()
        x = np.zeros([2])
        y = np.zeros([2])
        data.add_data_set(x,y)
        self.assertIsInstance(data.xdata,list, msg = 'xdata should be a list')
        
    def test_is_ydata_a_list(self):
        data = DataStructure.DataStructure()
        x = np.zeros([2])
        y = np.zeros([2])
        data.add_data_set(x,y)
        self.assertIsInstance(data.ydata,list, msg = 'ydata should be a list')
    
    def test_is_x_in_xdata_a_numpy_array_if_numpy_array_sent(self):
        data = DataStructure.DataStructure()
        x = np.zeros([2])
        y = np.zeros([2])
        data.add_data_set(x,y)
        self.assertIsInstance(data.xdata[0], np.ndarray, msg = 'xdata[0] should be a numpy array')
        
    def test_is_x_in_xdata_a_numpy_array_if_list_sent(self):
        data = DataStructure.DataStructure()
        x = [0]
        y = np.zeros([2])
        data.add_data_set(x,y)
        self.assertIsInstance(data.xdata[0], np.ndarray, msg = 'xdata[0] should be a numpy array')
        
    def test_is_y_in_ydata_a_numpy_array_if_numpy_array_sent(self):
        data = DataStructure.DataStructure()
        x = np.zeros([2])
        y = np.zeros([2])
        data.add_data_set(x,y)
        self.assertIsInstance(data.ydata[0], np.ndarray, msg = 'ydata[0] should be a numpy array')
        
    def test_is_y_in_ydata_a_numpy_array_if_list_sent(self):
        data = DataStructure.DataStructure()
        x = [0]
        y = [0]
        data.add_data_set(x,y)
        self.assertIsInstance(data.ydata[0], np.ndarray, msg = 'ydata[0] should be a numpy array')
    
    def test_is_y_in_ydata_a_numpy_array_if_element_sent(self):
        data = DataStructure.DataStructure()
        x = [0]
        y = 0
        data.add_data_set(x,y)
        self.assertIsInstance(data.ydata[0], np.ndarray, msg = 'ydata[0] should be a numpy array')
    
    def test_is_x_in_xdata_a_numpy_array_if_numpy_array_of_size_1(self):
        data = DataStructure.DataStructure()
        x = np.zeros(2)
        y = np.zeros([2])
        data.add_data_set(x,y)
        self.assertIsInstance(data.xdata[0], np.ndarray, msg = 'xdata[0] should be a numpy array')
        
    def test_does_shape_match_y_shape(self):
        data = DataStructure.DataStructure()
        x = np.zeros([2])
        y = np.zeros([2])
        data.add_data_set(x,y)
        self.assertEqual(data.shape[0], data.ydata[0].shape, msg = 'shapes should match')
        
    def test_2d_y_output_2d_shape(self):
        data = DataStructure.DataStructure()
        x = np.zeros([2])
        y = np.zeros([2,2])
        data.add_data_set(x,y)
        self.assertEqual(len(data.shape[0]), len(data.ydata[0].shape), msg = 'lengths should match')
        
# --------------------------
# get_number_of_batches
# --------------------------
class Get_Number_Of_Batches(unittest.TestCase):
    def test_does_nbatch_equal_length_of_shape_for_1d_y(self):
        data = DataStructure.DataStructure()
        x = np.zeros([2])
        y = np.zeros([2])
        data.add_data_set(x,y)
        data.get_number_of_batches()
        self.assertEqual(data.nbatch,1, msg = 'nbatch should match number of calls to ''add_data_set''')
        
    def test_does_nbatch_match_for_1d_y_with_multiple_data_sets(self):
        data = DataStructure.DataStructure()
        x = np.zeros([2])
        y = np.zeros([2])
        data.add_data_set(x,y)
        data.add_data_set(x,y)
        data.get_number_of_batches()
        self.assertEqual(data.nbatch,2, msg = 'nbatch should match number of calls to ''add_data_set''')
    
    def test_does_nbatch_match_for_1d_y_with_multiple_data_sets_with_different_size(self):
        data = DataStructure.DataStructure()
        x = np.zeros([2])
        y = np.zeros([2])
        data.add_data_set(x,y)
        y = np.zeros([2,2])
        data.add_data_set(x,y)
        data.get_number_of_batches()
        self.assertEqual(data.nbatch,2, msg = 'nbatch should match number of calls to ''add_data_set''')
    
    def test_does_nbatch_equal_length_of_shape_for_1d_y_v2(self):
        data = DataStructure.DataStructure()
        x = np.zeros([2,])
        y = np.zeros([2,])
        data.add_data_set(x,y)
        data.get_number_of_batches()
        self.assertEqual(data.nbatch,1, msg = 'nbatch should match number of calls to ''add_data_set''')
    
    def test_does_nbatch_equal_length_of_shape_for_2d_y(self):
        data = DataStructure.DataStructure()
        x = np.zeros([2,])
        y = np.zeros([2,2])
        data.add_data_set(x,y)
        data.get_number_of_batches()
        self.assertEqual(data.nbatch, 1, msg = 'nbatch should match number of calls to ''add_data_set''')
        
# --------------------------
# get_number_of_data_sets
# --------------------------
class Get_Number_Of_Data_Sets(unittest.TestCase):
    def test_does_single_data_set_match(self):
        data = DataStructure.DataStructure()
        x = np.zeros([2])
        y = np.zeros([2])
        data.add_data_set(x,y)
        data.get_number_of_data_sets()
        self.assertEqual(data.ndatasets, 1, msg = 'ndatasets should match total number of columns of elements of ydata')
        
    def test_does_double_data_set_match(self):
        data = DataStructure.DataStructure()
        x = np.zeros([2])
        y = np.zeros([2])
        data.add_data_set(x,y)
        data.add_data_set(x,y)
        data.get_number_of_data_sets()
        self.assertEqual(data.ndatasets, 2, msg = 'ndatasets should match total number of columns of elements of ydata')
        
    def test_does_double_data_set_match_with_2d_sets(self):
        data = DataStructure.DataStructure()
        x = np.zeros([2])
        y = np.zeros([2,2])
        data.add_data_set(x,y)
        data.add_data_set(x,y)
        data.get_number_of_data_sets()
        self.assertEqual(data.ndatasets, 4, msg = 'ndatasets should match total number of columns of elements of ydata')
     
    def test_does_double_data_set_match_with_2d_sets_of_different_size(self):
        data = DataStructure.DataStructure()
        x = np.zeros([2])
        y = np.zeros([2,2])
        data.add_data_set(x,y)
        y = np.zeros([3,4])
        data.add_data_set(x,y)
        data.get_number_of_data_sets()
        self.assertEqual(data.ndatasets, 6, msg = 'ndatasets should match total number of columns of elements of ydata')
     
    def test_does_double_data_set_match_with_different_sizes(self):
        data = DataStructure.DataStructure()
        x = np.zeros([2])
        y = np.zeros([2])
        data.add_data_set(x,y)
        y = np.zeros([2,2])
        data.add_data_set(x,y)
        data.get_number_of_data_sets()
        self.assertEqual(data.ndatasets, 3, msg = 'ndatasets should match total number of columns of elements of ydata')
        
        
# --------------------------
# get_number_of_observations
# --------------------------
class Get_Number_Of_Observations(unittest.TestCase):
    def test_number_of_observations_match_sum_of_n(self):
        data = DataStructure.DataStructure()
        x = np.zeros([2])
        y = np.zeros([2])
        data.add_data_set(x,y)
        nds = data.get_number_of_observations()
        self.assertEqual(nds[0], 2, msg = 'total number of observations is number of rows in y')
        
    def test_number_of_observations_match_sum_of_n_with_2d_y(self):
        data = DataStructure.DataStructure()
        x = np.zeros([2])
        y = np.zeros([2,2])
        data.add_data_set(x,y)
        nds = data.get_number_of_observations()
        self.assertEqual(nds[0], 2, msg = 'total number of observations is number of rows in y')
        
    def test_number_of_observations_match_sum_of_n_with_multiple_sets(self):
        data = DataStructure.DataStructure()
        x = np.zeros([2])
        y = np.zeros([2])
        data.add_data_set(x,y)
        data.add_data_set(x,y)
        nds = data.get_number_of_observations()
        self.assertEqual(nds[0], 4, msg = 'total number of observations is number of rows in y - summed over all sets of y')
        
    def test_number_of_observations_match_sum_of_n_with_multiple_sets_with_2d_y(self):
        data = DataStructure.DataStructure()
        x = np.zeros([2])
        y = np.zeros([2])
        data.add_data_set(x,y)
        y = np.zeros([2,2])
        data.add_data_set(x,y)
        nds = data.get_number_of_observations()
        self.assertEqual(nds[0], 4, msg = 'total number of observations is number of rows in y - summed over all sets of y')
        
    def test_number_of_observations_match_sum_of_n_with_multiple_2d_sets(self):
        data = DataStructure.DataStructure()
        x = np.zeros([2])
        y = np.zeros([1,2])
        data.add_data_set(x,y)
        y = np.zeros([2,2])
        data.add_data_set(x,y)
        nds = data.get_number_of_observations()
        self.assertEqual(nds[0], 3, msg = 'total number of observations is number of rows in y - summed over all sets of y')


#!/usr/bin/env python2
# -*- coding: utf-8 -*-
"""
Created on Thu Jan 18 10:42:07 2018

@author: prmiles
"""
# import required packages
import numpy as np
from ..structures.ParameterSet import ParameterSet
from .utilities import sample_candidate_from_gaussian_proposal
from .utilities import is_sample_outside_bounds, set_outside_bounds
from .utilities import acceptance_test

class DelayedRejection:
    '''
    Delayed Rejection (DR) algorithm based on :cite:`haario2006dram`.

    Attributes:
        * :meth:`~run_delayed_rejection`
        * :meth:`~initialize_next_metropolis_step`
        * :meth:`~alphafun`        
    '''
    # -------------------------------------------
    def run_delayed_rejection(self, old_set, new_set, RDR, ntry, parameters, invR, sosobj, priorobj):
        '''
        Perform delayed rejection step - occurs in standard metropolis is not accepted.

        Args:
            * **old_set** (:class:`~.ParameterSet`): Features of :math:`q^{k-1}`
            * **new_set** (:class:`~.ParameterSet`): Features of :math:`q^*`
            * **RDR** (:class:`~numpy.ndarray`): Cholesky decomposition of parameter covariance matrix for DR steps
            * **ntry** (:py:class:`int`): Number of DR steps to perform until rejection
            * **parameters** (:class:`~.ModelParameters`): Model parameters
            * **invR** (:class:`~numpy.ndarray`): Inverse Cholesky decomposition matrix
            * **sosobj** (:class:`~.SumOfSquares`): Sum-of-Squares function
            * **priorobj** (:class:`~.PriorFunction`): Prior function

        Returns:
            * **accept** (:py:class:`int`): 0 - reject, 1 - accept
            * **out_set** (:class:`~.ParameterSet`): If accept == 1, then latest DR set; Else, :math:`q^k=q^{k-1}`
            * **outbound** (:py:class:`int`): 1 - rejected due to sampling outside of parameter bounds
        '''
        # create trypath
        trypath = [old_set, new_set]
        itry = 1; # dr step index
        accept = False # initialize acceptance criteria
        while accept is False and itry < ntry:
            itry += 1 # update dr step index
            # initialize next step parameter set
            next_set = self.initialize_next_metropolis_step(npar = parameters.npar, old_theta = old_set.theta, sigma2 = new_set.sigma2, RDR = RDR[itry-1])

            # Reject points outside boundaries
            outsidebounds = is_sample_outside_bounds(next_set.theta, parameters._lower_limits[parameters._parind[:]], parameters._upper_limits[parameters._parind[:]])
            if outsidebounds is True:
                next_set, outbound = set_outside_bounds(next_set = next_set)
                trypath.append(next_set)
                out_set = old_set
                continue # return to beginning of while loop
                    
            # Evaluate new proposals
            outbound = 0
            next_set.ss = sosobj.evaluate_sos_function(next_set.theta)
            next_set.prior = priorobj.evaluate_prior(next_set.theta)
            trypath.append(next_set) # add set to trypath
            alpha = self.alphafun(trypath, invR)
            trypath[-1].alpha = alpha # add propability ratio
                           
            # check results of delayed rejection
            accept = acceptance_test(alpha = alpha)
            out_set = update_set_based_on_acceptance(accept, old_set = old_set, next_set = next_set)
            self.iacce[itry - 1] += accept # if accepted, adds 1, if not, adds 0
                
        return accept, out_set, outbound
    # -------------------------------------------
    @classmethod
    def initialize_next_metropolis_step(cls, npar, old_theta, sigma2, RDR):
        '''
        Take metropolis step according to DR

        Args:
            * **npar** (:py:class:`int`): Number of parameters
            * **old_theta** (:class:`~numpy.ndarray`): `q^{k-1}`
            * **sigma2** (:py:class:`float`): Observation error variance
            * **RDR** (:class:`~numpy.ndarray`): Cholesky decomposition of parameter covariance matrix for DR steps
            * **itry** (:py:class:`int`): DR step counter

        Returns:
            * **next_set** (:class:`~.ParameterSet`): New proposal set
            * **u** (:class:`numpy.ndarray`): Numbers sampled from standard normal distributions (:code:`u.shape = (1,npar)`)
        '''
        next_set = ParameterSet()
        next_set.theta, u = sample_candidate_from_gaussian_proposal(npar = npar, oldpar = old_theta, R = RDR)
        next_set.sigma2 = sigma2
        return next_set
    # -------------------------------------------
    def _initialize_dr_metrics(self, options):
        '''
        Initialize counting metrics for delayed rejection algorithm.

        Args:
            * **options** (:class:`~.SimulationOptions`): MCMC simulation options
        '''
        self.iacce = np.zeros(options.ntry, dtype = int)
        self.dr_step_counter = 0

    # -------------------------------------------
    def alphafun(self, trypath, invR):
        '''
        Calculate likelihood according to DR

        Args:
            * **trypath** (:py:class:`list`): Sequence of DR steps
            * **invR** (:class:`~numpy.ndarray`): Inverse Cholesky decomposition matrix

        Returns:
            * **alpha** (:py:class:`float`): Result of likelihood function according to delayed rejection
        '''
        self.dr_step_counter += 1
        stage = len(trypath) - 1 # The stage we're in, elements in trypath - 1
        # recursively compute past alphas
        a1 = 1.0 # initialize
        a2 = 1.0 # initialize
        for kk in range(0,stage-1):
            tmp1 = self.alphafun(trypath[0:(kk+2)], invR)
            a1 = a1*(1 - tmp1)
            tmp2 = self.alphafun(trypath[stage:stage-kk-2:-1], invR)
            a2 = a2*(1 - tmp2)
            if a2 == 0: # we will come back with prob 1
                alpha = np.zeros(1)
                return alpha
            
        y = log_posterior_ratio(trypath[0], trypath[-1])
        
        for kk in range(stage):
            y = y + nth_stage_log_proposal_ratio(kk, trypath, invR)
            
        alpha = min(np.ones(1), np.exp(y)*a2*(a1**(-1)))
        
        return alpha

# -------------------------------------------
def nth_stage_log_proposal_ratio(iq, trypath, invR):
    '''
    Gaussian nth stage log proposal ratio.

    Logarithm of :math:`q_i(y_n,...,y_{n-j})/q_i(x,y_1,...,y_j)`

    Args:
        * **iq** (:py:class:`int`): Stage number.
        * **trypath** (:py:class:`list`): Sequence of DR steps
        * **invR** (:class:`~numpy.ndarray`): Inverse Cholesky decomposition matrix

    Returns:
        * **zq** (:py:class:`float`): Logarithm of Gaussian nth stage proposal ratio.
    '''
    stage = len(trypath) - 1 - 1 # - 1, iq;
    if stage == iq: # shift index due to 0-indexing
        zq = np.zeros(1) # we are symmetric
    else:
        iR = invR[iq] # proposal^(-1/2)
        y1, y2, y3, y4 = extract_state_elements(iq = iq, stage = stage, trypath = trypath)
        zq = -0.5*((np.linalg.norm(np.dot(y4-y3, iR)))**2 - (np.linalg.norm(np.dot(y2-y1, iR)))**2)
        
    return zq

# -------------------------------------------
def extract_state_elements(iq, stage, trypath):
    '''
    Extract elements from tried paths.

    Args:
        * **iq** (:py:class:`int`): Stage number.
        * **stage** (:py:class:`int`): Number of stages - 2
        * **trypath** (:py:class:`list`): Sequence of DR steps
    '''
    y1 = trypath[0].theta
    y2 = trypath[iq + 1].theta # check index
    y3 = trypath[stage + 1].theta
    y4 = trypath[stage - iq].theta
    return y1, y2, y3, y4
# -------------------------------------------
def log_posterior_ratio(x1, x2):
    '''
    Calculate the logarithm of the posterior ratio.

    Args:
        * **x1** (:class:`~.ParameterSet`): Old set - :math:`q^{k-1}`
        * **x2** (:class:`~.ParameterSet`): New set - :math:`q^*`

    Returns:
        * **zq** (:py:class:`float`): Logarithm of posterior ratio.
    '''
    zq = -0.5*(sum((x2.ss*(x2.sigma2**(-1.0)) - x1.ss*(x1.sigma2**(-1.0)))) + x2.prior - x1.prior)
    return sum(zq)
    
# -------------------------------------------
def update_set_based_on_acceptance(accept, old_set, next_set):
    '''
    Define output set based on acceptance

    .. math::

        & \\text{If}~u_{\\alpha} <~\\alpha, \\

        & \\quad \\text{Set}~q^k = q^*,~SS_{q^k} = SS_{q^*} \\

        & \\text{Else} \\

        & \\quad \\text{Set}~q^k = q^{k-1},~SS_{q^k} = SS_{q^{k-1}}

    Args:
        * **accept** (:py:class:`int`): 0 - reject, 1 - accept
        * **old_set** (:class:`~.ParameterSet`): Features of :math:`q^{k-1}`
        * **next_set** (:class:`~.ParameterSet`): New proposal set

    Returns:
        * **out_set** (:class:`~.ParameterSet`): If accept == 1, then latest DR set; Else, :math:`q^k=q^{k-1}`
    '''
    if accept is True:
        out_set = next_set
    else:
        out_set = old_set
    return out_set


#!/usr/bin/env python3
# -*- coding: utf-8 -*-
"""
Created on Thu May 31 06:55:18 2018

@author: prmiles
"""

from pymcmcstat.procedures.CovarianceProcedures import CovarianceProcedures
import unittest
import numpy as np
import test.general_functions as gf

# --------------------------
class InitializeCP(unittest.TestCase):

    def test_init_CP(self):
        CP = CovarianceProcedures()
        self.assertTrue(hasattr(CP, 'description'))
        
# --------------------------
class UpdateCovarianceFromAdaptation(unittest.TestCase):

    def test_update_cov(self):
        __, options, parameters, __ = gf.setup_mcmc()
        CP = CovarianceProcedures()
        CP._initialize_covariance_settings(parameters = parameters, options = options)
        
        check = {
        'R': np.random.random_sample(size = (2,2)),
        'covchain': np.random.random_sample(size = (2,2)),
        'meanchain': np.random.random_sample(size = (1,2)),
        'wsum': np.random.random_sample(size = (2,1)),
        'last_index_since_adaptation': 0,
        'iiadapt': 100
        }
        
        CP._update_covariance_from_adaptation(**check)
        CPD = CP.__dict__
        items = ['last_index_since_adaptation', 'iiadapt']
        for __, ai in enumerate(items):
            self.assertEqual(CPD[str('_{}'.format(ai))], check[ai], msg = str('{}: {} != {}'.format(ai, CPD[str('_{}'.format(ai))], check[ai])))
            
        array_items = ['R', 'covchain', 'meanchain', 'wsum']
        for __, ai in enumerate(array_items):
            self.assertTrue(np.array_equal(CPD[str('_{}'.format(ai))], check[ai]), msg = str('{}: {} != {}'.format(ai, CPD[str('_{}'.format(ai))], check[ai])))
            
# --------------------------
class UpdateCovarianceFromDelayedRejection(unittest.TestCase):

    def test_update_cov(self):
        __, options, parameters, data = gf.setup_mcmc()
        CP = CovarianceProcedures()
        CP._initialize_covariance_settings(parameters = parameters, options = options)
        
        check = {
        'RDR': np.random.random_sample(size = (2,2)),
        'invR': np.random.random_sample(size = (2,2)),
        }
        
        CP._update_covariance_for_delayed_rejection_from_adaptation(**check)
        CPD = CP.__dict__
        array_items = ['RDR', 'invR']
        for __, ai in enumerate(array_items):
            self.assertTrue(np.array_equal(CPD[str('_{}'.format(ai))], check[ai]), msg = str('{}: {} != {}'.format(ai, CPD[str('_{}'.format(ai))], check[ai])))
            
# --------------------------
class UpdateCovarianceSettings(unittest.TestCase):

    def test_update_cov_wsum_none(self):
        model, options, parameters, data = gf.setup_mcmc()
        CP = CovarianceProcedures()
        CP._initialize_covariance_settings(parameters = parameters, options = options)
        
        theta = np.array([2., 5.])
        CP._wsum = None
        CP._covchain = 1
        CP._meanchain = 2
        CP._qcov = 0
        CP._update_covariance_settings(parameter_set = theta)
        
        CPD = CP.__dict__
        self.assertEqual(CPD['_covchain'], 1, msg = '_covchain unchanged.')
        self.assertEqual(CPD['_meanchain'], 2, msg = '_meanchain unchanged.')
        
    def test_update_cov_wsum_not_none(self):
        model, options, parameters, data = gf.setup_mcmc()
        CP = CovarianceProcedures()
        CP._initialize_covariance_settings(parameters = parameters, options = options)
        
        theta = np.array([2., 5.])
        CP._wsum = 10
        CP._covchain = 1
        CP._meanchain = 2
        CP._qcov = 0
        CP._update_covariance_settings(parameter_set = theta)
        
        CPD = CP.__dict__
        self.assertEqual(CPD['_covchain'], 0, msg = '_covchain = _qcov.')
        self.assertTrue(np.array_equal(CPD['_meanchain'], theta), msg = '_meanchain = parameter_set.')

# -------------------------------------------
class SetupRBasedOnVariances(unittest.TestCase):
    def test_one_variance(self):
        model, options, parameters, data = gf.setup_mcmc()
        CP = CovarianceProcedures()
        CP._qcov = np.atleast_2d([0.2])
        CP._CovarianceProcedures__setup_R_based_on_variances(parind = [0])
        self.assertEqual(CP._R, np.sqrt(0.2), msg = 'Expect sqrt of variance')
        
    def test_three_variances(self):
        model, options, parameters, data = gf.setup_mcmc()
        CP = CovarianceProcedures()
        CP._qcov = np.atleast_2d([0.2, 0.3, 0.4])
        CP._CovarianceProcedures__setup_R_based_on_variances(parind = [0, 1, 2])
        self.assertTrue(np.array_equal(CP._R, np.diagflat(np.sqrt([0.2, 0.3, 0.4]))), msg = str('Expect sqrt of variances: {}'.format(CP._R)))
        self.assertTrue(np.array_equal(CP._qcovorig, np.diagflat([0.2, 0.3, 0.4])), msg = 'Arrays should match')
        
# -------------------------------------------
class SetupRBasedOnCovarianceMatrix(unittest.TestCase):
    def test_2x2_cov_mtx(self):
        model, options, parameters, data = gf.setup_mcmc()
        CP = CovarianceProcedures()
        CP._qcov = np.atleast_2d([[0.2, 0.1],[0.1,0.3]])
        CP._CovarianceProcedures__setup_R_based_on_covariance_matrix(parind = [0, 1])
        self.assertTrue(np.array_equal(CP._R, np.linalg.cholesky(np.atleast_2d([[0.2, 0.1],[0.1,0.3]])).T), msg = 'Expect sqrt of variance')
        self.assertTrue(np.array_equal(CP._qcovorig, np.atleast_2d([[0.2, 0.1],[0.1,0.3]])), msg = 'Expect sqrt of variance')
        
    def test_3x3_cov_mtx(self):
        model, options, parameters, data = gf.setup_mcmc()
        CP = CovarianceProcedures()
        testmtx = np.atleast_2d([[0.2, 0.01, 0.05],[0.01,0.3,0.024],[0.05,0.024,0.04]])
        CP._qcov = testmtx
        CP._CovarianceProcedures__setup_R_based_on_covariance_matrix(parind = [0, 1, 2])
        self.assertTrue(np.array_equal(CP._R, np.linalg.cholesky(testmtx).T), msg = 'Expect sqrt of variance')
        self.assertTrue(np.array_equal(CP._qcovorig, testmtx), msg = 'Expect sqrt of variance')
        
    def test_3x3_cov_mtx_with_non_sample(self):
        model, options, parameters, data = gf.setup_mcmc()
        CP = CovarianceProcedures()
        testmtx = np.atleast_2d([[0.2, 0.01, 0.05],[0.01,0.3,0.024],[0.05,0.024,0.04]])
        parind = [0, 2]
        CP._qcov = testmtx
        CP._CovarianceProcedures__setup_R_based_on_covariance_matrix(parind = parind)
        self.assertTrue(np.array_equal(CP._R, np.linalg.cholesky(testmtx[np.ix_(parind,parind)]).T), msg = 'Expect sqrt of variance')
        self.assertTrue(np.array_equal(CP._qcovorig, testmtx), msg = 'Expect sqrt of variance')

# -------------------------------------------
class SetupNoAdaptIndex(unittest.TestCase):
    def test_noadapt_empty(self):
        model, options, parameters, data = gf.setup_mcmc()
        CP = CovarianceProcedures()
        CP._initialize_covariance_settings(parameters = parameters, options = options)
        CP._no_adapt_index = []
        CP._CovarianceProcedures__setup_no_adapt_index(noadaptind = [], parind = [0, 1])
        self.assertTrue(np.array_equal(CP._no_adapt_index, np.zeros([2],dtype=bool)), msg = 'Arrays should match')
        
    def test_noadapt_not_empty(self):
        model, options, parameters, data = gf.setup_mcmc()
        CP = CovarianceProcedures()
        CP._initialize_covariance_settings(parameters = parameters, options = options)
        CP._no_adapt_index = []
        CP._CovarianceProcedures__setup_no_adapt_index(noadaptind = [1], parind = [0, 1])
        self.assertTrue(np.array_equal(CP._no_adapt_index, np.array([0, 1],dtype=bool)), msg = 'Arrays should match')
        CP._CovarianceProcedures__setup_no_adapt_index(noadaptind = [0,2], parind = [0, 1, 2])
        self.assertTrue(np.array_equal(CP._no_adapt_index, np.array([1, 0, 1],dtype=bool)), msg = 'Arrays should match')
        
# -------------------------------------------
class DisplayCovarianceSettings(unittest.TestCase):
    
    def test_print_these_none(self):
        model, options, parameters, data = gf.setup_mcmc()
        CP = CovarianceProcedures()
        CP._initialize_covariance_settings(parameters = parameters, options = options)
        
        print_these = CP.display_covariance_settings(print_these = None)
        self.assertEqual(print_these, ['qcov', 'R', 'RDR', 'invR', 'last_index_since_adaptation', 'covchain'], msg = 'Default print keys')
        
    def test_print_these_not_none(self):
        model, options, parameters, data = gf.setup_mcmc()
        CP = CovarianceProcedures()
        CP._initialize_covariance_settings(parameters = parameters, options = options)
        
        print_these = ['qcov', 'R', 'RDR', 'invR', 'last_index_since_adaptation', 'covchain']
        for __, ptii in enumerate(print_these):
            self.assertEqual(CP.display_covariance_settings(print_these=[ptii]), [ptii], msg = 'Specified print keys')
# -------------------------------------------


#!/usr/bin/env python3
# -*- coding: utf-8 -*-
"""
Created on Mon May 14 06:24:12 2018

@author: prmiles
"""
import numpy as np
from scipy.interpolate import interp1d
from scipy import pi,sin,cos
import sys
import math

def generate_subplot_grid(nparam = 2):
    '''
    Generate subplot grid.

    For example, if `nparam` = 2, then the subplot will have 2 rows and 1 column.

    Args:
        * **nparam** (:py:class:`int`): Number of parameters

    Returns:
        * **ns1** (:py:class:`int`): Number of rows in subplot
        * **ns2** (:py:class:`int`): Number of columns in subplot
    '''
    ns1 = math.ceil(math.sqrt(nparam))
    ns2 = round(math.sqrt(nparam))
    return ns1, ns2

def generate_names(nparam, names):
    '''
    Generate parameter name set.

    For example, if `nparam` = 4, then the generated names are::

        names = ['p_{0}', 'p_{1}', 'p_{2}', 'p_{3}']

    Args:
        * **nparam** (:py:class:`int`): Number of parameter names to generate
        * **names** (:py:class:`list`): Names of parameters provided by user

    Returns:
        * **names** (:py:class:`list`): List of strings - parameter names
    '''
    # Check if names defined
    if names == None:
        names = generate_default_names(nparam)

    # Check if enough names defined
    if len(names) != nparam:
        names = extend_names_to_match_nparam(names, nparam)
    return names

def setup_plot_features(nparam, names, figsizeinches):
    '''
    Setup plot features.

    Args:
        * **nparam** (:py:class:`int`): Number of parameters
        * **names** (:py:class:`list`): Names of parameters provided by user
        * **figsizeinches** (:py:class:`list`): [Width, Height]

    Returns:
        * **ns1** (:py:class:`int`): Number of rows in subplot
        * **ns2** (:py:class:`int`): Number of columns in subplot
        * **names** (:py:class:`list`): List of strings - parameter names
        * **figsizeiches** (:py:class:`list`): [Width, Height]
    '''
    ns1, ns2 = generate_subplot_grid(nparam = nparam)

    names = generate_names(nparam = nparam, names = names)
    
    if figsizeinches is None:
        figsizeinches = [5,4]
        
    return ns1, ns2, names, figsizeinches

def generate_default_names(nparam):
    '''
    Generate generic parameter name set.

    For example, if `nparam` = 4, then the generated names are::

        names = ['p_{0}', 'p_{1}', 'p_{2}', 'p_{3}']

    Args:
        * **nparam** (:py:class:`int`): Number of parameter names to generate

    Returns:
        * **names** (:py:class:`list`): List of strings - parameter names
    '''
    names = []
    for ii in range(nparam):
        names.append(str('$p_{{{}}}$'.format(ii)))
    return names

def extend_names_to_match_nparam(names, nparam):
    '''
    Append names to list using default convention
    until length of names matches number of parameters.

    For example, if `names = ['name_1', 'name_2']` and `nparam = 4`, then
    two additional names will be appended to the `names` list.
    E.g.,::

        names = ['name_1', 'name_2', 'p_{2}', 'p_{3}']

    Args:
        * **names** (:py:class:`list`): Names of parameters provided by user
        * **nparam** (:py:class:`int`): Number of parameter names to generate

    Returns:
        * **names** (:py:class:`list`): List of strings - extended list of parameter names
    '''
    if names is None:
        names = []
        
    n0 = len(names)
    for ii in range(n0,nparam):
        names.append(str('$p_{{{}}}$'.format(ii)))
    return names

# --------------------------------------------
def make_x_grid(x, npts = 100):
    '''
    Generate x grid based on extrema.

    1. If `len(x) > 200`, then generates grid based on difference
    between the max and min values in the array.

    2. Otherwise, the grid is defined with respect to the array
    mean plus or minus four standard deviations.

    Args:
        * **x** (:class:`~numpy.ndarray`): Array of points
        * **npts** (:py:class:`int`): Number of points to use in generated grid

    Returns:
        * Uniformly spaced array of points with shape :code:`=(npts,1)`. (:class:`~numpy.ndarray`)
    '''
    xmin = min(x)
    xmax = max(x)
    xxrange = xmax-xmin
    if len(x) > 200:
        x_grid=np.linspace(xmin-0.08*xxrange,xmax+0.08*xxrange,npts)
    else:
        x_grid=np.linspace(np.mean(x)-4*np.std(x, ddof=1),np.mean(x)+4*np.std(x, ddof=1),npts)
    return x_grid.reshape(x_grid.shape[0],1) # returns 1d column vector

# --------------------------------------------
# see MASS 2nd ed page 181.
def iqrange(x):
    '''
    Interquantile range of each column of x.
    
    Args:
        * **x** (:class:`~numpy.ndarray`): Array of points.

    Returns:
        * (:class:`~numpy.ndarray`): Interquantile range - single element array, `q3 - q1`.
    '''
    nr, nc = x.shape
    if nr == 1: # make sure it is a column vector
        x = x.reshape(nc,nr)
        nr = nc
        nc = 1
    
    # sort
    x.sort()
    
    i1 = math.floor((nr + 1)/4)
    i3 = math.floor(3/4*(nr+1))
    f1 = (nr+1)/4-i1
    f3 = 3/4*(nr+1)-i3
    q1 = (1-f1)*x[int(i1),:] + f1*x[int(i1)+1,:]
    q3 = (1-f3)*x[int(i3),:] + f3*x[int(i3)+1,:]
    return q3-q1
    
def gaussian_density_function(x, mu = 0, sigma2 = 1):
    '''
    Standard normal/Gaussian density function.
    
    Args:
        * **x** (:py:class:`float`): Value of which to calculate probability.
        * **mu** (:py:class:`float`): Mean of Gaussian distribution.
        * **sigma2** (:py:class:`float`): Variance of Gaussian distribution.

    Returns:
        * **y** (:py:class:`float`): Likelihood of `x`.
    '''
    y = 1/math.sqrt(2*math.pi*sigma2)*math.exp(-0.5*(x-mu)**2/sigma2)
    return y

def scale_bandwidth(x):
    '''
    Scale bandwidth of array.
    
    Args:
        * **x** (:class:`~numpy.ndarray`): Array of points - column of chain.

    Returns:
        * **s** (:class:`~numpy.ndarray`): Scaled bandwidth - single element array.
    '''
    n = len(x)
    if iqrange(x) <= 0:
        s = 1.06*np.array([np.std(x, ddof=1)*n**(-1/5)])
    else:
        s = 1.06*np.array([min(np.std(x, ddof=1),iqrange(x)/1.34)*n**(-1/5)])
    return s

# --------------------------------------------
def generate_ellipse(mu, cmat, ndp = 100):
    '''
    Generates points for a probability contour ellipse

    Args:
        * **mu** (:class:`~numpy.ndarray`): Mean values
        * **cmat** (:class:`~numpy.ndarray`): Covariance matrix
        * **npd** (:py:class:`int`): Number of points to generate

    Returns:
        * **x** (:class:`~numpy.ndarray`): x-points
        * **y** (:class:`~numpy.ndarray`): y-points
    '''
    
    # check shape of covariance matrix
    if cmat.shape != (2,2):
        sys.exit('covariance matrix must be 2x2')
    
    if check_symmetric(cmat) is not True:
        sys.exit('covariance matrix must be symmetric')
        
    
    # define t space
    t = np.linspace(0, 2*pi, ndp)
    
    pdflag, R = is_semi_pos_def_chol(cmat)
    if pdflag is False:
        sys.exit('covariance matrix must be positive definite')
    
    x = mu[0] + R[0,0]*cos(t)
    y = mu[1] + R[0,1]*cos(t) + R[1,1]*sin(t)
    
    return x, y

def check_symmetric(a, tol=1e-8):
    '''
    Check if array is symmetric by comparing with transpose.
    
    Args:
        * **a** (:class:`~numpy.ndarray`): Array to test.
        * **tol** (:py:class:`float`): Tolerance for testing equality.

    Returns:
        * (:py:class:`bool`): True -> symmetric, False -> not symmetric.
    '''
    return np.allclose(a, a.T, atol = tol)

def is_semi_pos_def_chol(x):
    '''
    Check if matrix is semi positive definite by calculating Cholesky decomposition.

    Args:
        * **x** (:class:`~numpy.ndarray`): Matrix to check

    Returns:
        * If matrix is `not` semi positive definite return :code:`False, None`
        * If matrix is semi positive definite return :code:`True` and the Upper triangular form of the Cholesky decomposition matrix.
    '''
    c = None
    try:
        c = np.linalg.cholesky(x)
        return True, c.transpose()
    except np.linalg.linalg.LinAlgError:
        return False, c
    
def append_to_nrow_ncol_based_on_shape(sh, nrow, ncol):
    '''
    Append to list based on shape of array

    Args:
        * **sh** (:py:class:`tuple`): Shape of array.
        * **nrow** (:py:class:`list`): List of number of rows
        * **ncol** (:py:class:`list`): List of number of columns

    Returns:
        * **nrow** (:py:class:`list`): List of number of rows
        * **ncol** (:py:class:`list`): List of number of columns
    '''
    if len(sh) == 1:
        nrow.append(sh[0])
        ncol.append(1)
    else:
        nrow.append(sh[0])
        ncol.append(sh[1])
    return nrow, ncol

# --------------------------------------------
def convert_flag_to_boolean(flag):
    '''
    Convert flag to boolean for backwards compatibility.

    Args:
        * **flag** (:py:class:`bool` or :py:class:`int`): Flag to specify something.

    Returns:
        * **flag** (:py:class:`bool`): Flag to converted to boolean.
    '''
    if flag is 'on':
        flag = True
    elif flag is 'off':
        flag = False
        
    return flag

# --------------------------------------------
def set_local_parameters(ii, local):
    '''
    Set local parameters based on tests.
    
    :Test 1:
        * `local == 0`
    :Test 2:
        * `local == ii`
    
    Args:
        * **ii** (:py:class:`int`): Index.
        * **local** (:class:`~numpy.ndarray`): Local flags.

    Returns:
        * **test** (:class:`~numpy.ndarray`): Array of Booleans indicated test results.
    '''
    # some parameters may only apply to certain batch sets
    test1 = local == 0
    test2 = local == ii
    test = test1 + test2
    return test.reshape(test.size,)

# --------------------------------------------
def empirical_quantiles(x, p = np.array([0.25, 0.5, 0.75])):
    '''
    Calculate empirical quantiles.

    Args:
        * **x** (:class:`~numpy.ndarray`): Observations from which to generate quantile.
        * **p** (:class:`~numpy.ndarray`): Quantile limits.

    Returns:
        * (:class:`~numpy.ndarray`): Interpolated quantiles.
    '''

    # extract number of rows/cols from np.array
    n = x.shape[0]
    # define vector valued interpolation function
    xpoints = range(n)
    interpfun = interp1d(xpoints, np.sort(x, 0), axis = 0)
    
    # evaluation points
    itpoints = (n-1)*p
    
    return interpfun(itpoints)

# --------------------------------------------    
def check_defaults(kwargs, defaults):
    '''
    Check if defaults are defined in kwargs
    
    Args:
        * **kwargs** (:py:class:`dict`): Keyword arguments.
        * **defaults** (:py:class:`dict`): Default settings.

    Returns:
        * **kwargs** (:py:class:`dict`): Updated keyword arguments with at least defaults set.
    '''
    for ii in defaults:
        if ii not in kwargs:
            kwargs[ii] = defaults[ii]
    return kwargs


#!/usr/bin/env python3
# -*- coding: utf-8 -*-
"""
Created on Wed May 30 08:02:45 2018

@author: prmiles
"""

from pymcmcstat.structures.ResultsStructure import ResultsStructure
from pymcmcstat.settings.SimulationOptions import SimulationOptions
from pymcmcstat.settings.ModelSettings import ModelSettings
from pymcmcstat.samplers.DelayedRejection import DelayedRejection
import test.general_functions as gf
import unittest
import numpy as np
import os
    
# -------------------
class SaveLoadJSONObject(unittest.TestCase):
    
    def test_dump_to_json_file(self):
        RS = ResultsStructure()
        tmpfile = gf.generate_temp_file(extension = 'json')
        results = {'model': 4, 'data': np.random.random_sample(size = (1000,2))}
        RS.save_json_object(results = results, filename = tmpfile)
        retres = RS.load_json_object(filename = tmpfile)
        self.assertEqual(retres['model'], results['model'], msg = 'Model set to 4')
        self.assertTrue(np.array_equal(retres['data'], results['data']), msg = 'Arrays should be equal')
        os.remove(tmpfile)

# -------------------
class DetermineFilename(unittest.TestCase):
    def test_resfilename_is_none(self):
        __, options, parameters, data, covariance, rejected, chain, s2chain, sschain = gf.setup_mcmc_case_dr()
        RS = ResultsStructure()
        RS.add_options(options = options)
        RS.results['simulation_options']['results_filename'] = None
        filename = RS.determine_filename(options = RS.results['simulation_options'])
        self.assertEqual(filename, str('{}{}{}'.format(RS.results['simulation_options']['datestr'],'_','mcmc_simulation.json')), msg = 'Filename matches')
        
    def test_resfilename_is_not_none(self):
        __, options, parameters, data, covariance, rejected, chain, s2chain, sschain = gf.setup_mcmc_case_dr()
        RS = ResultsStructure()
        RS.add_options(options = options)
        RS.results['simulation_options']['results_filename'] = 'test'
        filename = RS.determine_filename(options = RS.results['simulation_options'])
        self.assertEqual(filename, 'test', msg = 'Filename matches')
        
# -------------------
class AddBasic(unittest.TestCase):
    def test_addbasic_false(self):
        RS = ResultsStructure()
        self.assertFalse(RS.basic, msg = 'basic features not added to result structure')
        
    def test_addbasic_true(self):
        model, options, parameters, data, covariance, rejected, chain, s2chain, sschain = gf.setup_mcmc_case_dr()
        RS = ResultsStructure()
        RS.add_basic(nsimu = options.nsimu, covariance=covariance, parameters=parameters, rejected=rejected, simutime = 0.001, theta = chain[-1,:])
        self.assertTrue(RS.basic, msg = 'basic features added to result structure')
        self.assertTrue(np.array_equal(RS.results['theta'], np.array([0,0])), msg = 'Last elements of chain are zero')
        
    def test_addbasic_rejection(self):
        model, options, parameters, data, covariance, rejected, chain, s2chain, sschain = gf.setup_mcmc_case_dr()
        RS = ResultsStructure()
        RS.add_basic(nsimu = options.nsimu, covariance=covariance, parameters=parameters, rejected=rejected, simutime = 0.001, theta = chain[-1,:])
        self.assertEqual(RS.results['total_rejected'], 10*(options.nsimu**(-1)), msg = 'rejection reported as fraction of nsimu')
        self.assertEqual(RS.results['rejected_outside_bounds'], 2*(options.nsimu**(-1)), msg = 'rejection reported as fraction of nsimu')
        
    def test_addbasic_covariance(self):
        model, options, parameters, data, covariance, rejected, chain, s2chain, sschain = gf.setup_mcmc_case_dr()
        covariance._R[0,0] = 1.1
        covariance._R[0,1] = 2.3
        RS = ResultsStructure()
        RS.add_basic(nsimu = options.nsimu, covariance=covariance, parameters=parameters, rejected=rejected, simutime = 0.001, theta = chain[-1,:])
        self.assertTrue(np.array_equal(RS.results['R'], covariance._R), msg = 'Cholesky matches')
        self.assertTrue(np.array_equal(RS.results['qcov'], np.dot(covariance._R.transpose(),covariance._R)), msg = 'Covariance matches')

# -------------------
class AddDRAM(unittest.TestCase):
    def test_addbasic_false(self):
        model, options, parameters, data, covariance, rejected, chain, s2chain, sschain = gf.setup_mcmc_case_dr()
        DR = DelayedRejection()
        DR._initialize_dr_metrics(options)
        RS = ResultsStructure()
        self.assertFalse(RS.add_dram(drscale = options.drscale, RDR=covariance._RDR, total_rejected=rejected['total'], drsettings = DR), msg = 'basic features not added to result structure')
        
    def test_addbasic_true(self):
        model, options, parameters, data, covariance, rejected, chain, s2chain, sschain = gf.setup_mcmc_case_dr()
        covariance._RDR = np.random.random_sample(size = (2,2))
        DR = DelayedRejection()
        DR._initialize_dr_metrics(options)
        DR.dr_step_counter = 12000
        RS = ResultsStructure()
        RS.add_basic(nsimu = options.nsimu, covariance=covariance, parameters=parameters, rejected=rejected, simutime = 0.001, theta = chain[-1,:])
        self.assertTrue(RS.add_dram(drscale = options.drscale, RDR=covariance._RDR, total_rejected=rejected['total'], drsettings = DR), msg = 'basic features added to result structure')
        self.assertTrue(np.array_equal(RS.results['RDR'], covariance._RDR), msg = 'RDR matches')
        self.assertEqual(RS.results['alpha_count'], DR.dr_step_counter, msg = 'Alpha count matches dr step counter')
        
# -------------------
class AddUpdateSigma(unittest.TestCase):
    
    def test_updatesigma_true(self):
        updatesigma = True
        sigma2 = np.ones([2,1])
        S20 = np.ones([2,1])
        N0 = np.ones([2,1])
        RS = ResultsStructure()
        RS.add_updatesigma(updatesigma = updatesigma, sigma2 = sigma2, S20 = S20, N0 = N0)
        self.assertTrue(np.isnan(RS.results['sigma2']), msg = 'sigma2 -> np.nan')
        self.assertTrue(np.array_equal(RS.results['S20'], S20), msg = str('S20 -> {}'.format(S20)))
        self.assertTrue(np.array_equal(RS.results['N0'], N0), msg = str('N0 -> {}'.format(N0)))
        
        
    def test_updatesigma_false(self):
        updatesigma = False
        sigma2 = np.ones([2,1])
        S20 = np.ones([2,1])
        N0 = np.ones([2,1])
        RS = ResultsStructure()
        RS.add_updatesigma(updatesigma = updatesigma, sigma2 = sigma2, S20 = S20, N0 = N0)
        self.assertTrue(np.array_equal(RS.results['sigma2'], sigma2), msg = str('sigma2 -> {}'.format(sigma2)))
        self.assertTrue(np.isnan(RS.results['S20']), msg = 'S20 -> np.nan')
        self.assertTrue(np.isnan(RS.results['N0']), msg = 'N0 -> np.nan')
        
# -------------------
class AddArrays(unittest.TestCase):
    
    def test_add_rndnumseq(self):
        rnd = np.random.random_sample(size = (3,2))
        key = 'rndseq'
        RS = ResultsStructure()
        RS.add_random_number_sequence(rndseq = rnd)
        self.assertTrue(np.array_equal(RS.results[key], rnd),
                        msg = str('Expect equal arrays: {} != {}'.format(RS.results[key], rnd)))
        
    def test_add_chain(self):
        rnd = np.random.random_sample(size = (3,2))
        key = 'chain'
        RS = ResultsStructure()
        RS.add_chain(chain = rnd)
        self.assertTrue(np.array_equal(RS.results[key], rnd),
                        msg = str('Expect equal arrays: {} != {}'.format(RS.results[key], rnd)))
        
    def test_add_sschain(self):
        rnd = np.random.random_sample(size = (3,2))
        key = 'sschain'
        RS = ResultsStructure()
        RS.add_sschain(sschain = rnd)
        self.assertTrue(np.array_equal(RS.results[key], rnd),
                        msg = str('Expect equal arrays: {} != {}'.format(RS.results[key], rnd)))
        
    def test_add_s2chain(self):
        rnd = np.random.random_sample(size = (3,2))
        key = 's2chain'
        RS = ResultsStructure()
        RS.add_s2chain(s2chain = rnd)
        self.assertTrue(np.array_equal(RS.results[key], rnd),
                        msg = str('Expect equal arrays: {} != {}'.format(RS.results[key], rnd)))
    
# -------------------
class AddOptions(unittest.TestCase):
    
    def test_key_removal(self):
        options = SimulationOptions()
        options.define_simulation_options(doram = 3, nsimu = 500)
        RS = ResultsStructure()
        RS.add_options(options = options)
        self.assertEqual(RS.results['simulation_options']['nsimu'], options.nsimu, msg = str('nsimu = {}'.format(options.nsimu)))
        self.assertFalse('doram' in RS.results['simulation_options'].items(), msg = 'doram option should not be saved')
        
# -------------------
class AddModel(unittest.TestCase):
    
    def test_key_removal(self):
        model = ModelSettings()
        model.define_model_settings(N = 100, sos_function = 'hello world')
        RS = ResultsStructure()
        RS.add_model(model = model)
        self.assertEqual(RS.results['model_settings']['N'], model.N, msg = str('N = {}'.format(model.N)))
        self.assertFalse('sos_function' in RS.results['model_settings'].items(), msg = 'sos_function setting should not be saved')
        
# -------------------
class AddTimeStats(unittest.TestCase):
    
    def test_add_time_stats(self):
        key = 'time [mh, dr, am]'
        mtime = 0.01
        drtime = 0.10
        adtime = 0.05
        RS = ResultsStructure()
        RS.add_time_stats(mtime = mtime, drtime = drtime, adtime = adtime)
        self.assertEqual(RS.results[key], [mtime, drtime, adtime], msg = 'Lists should be equal.')
        
# -------------------
class AddPrior(unittest.TestCase):
    
    def test_add_prior(self):
        RS = ResultsStructure()
        mu = 0.0
        sigma = 1.0
        priorfun = 'priorfun'
        priortype = 1
        priorpars = 3
        RS.add_prior(mu = mu, sig = sigma, priorfun = priorfun, priorpars = priorpars, priortype = priortype)
        self.assertEqual(RS.results['prior'], [mu, sigma], msg = str('Expected [{},{}]'.format(mu, sigma)))
        self.assertEqual(RS.results['priorfun'], priorfun, msg = str('Expected {}'.format(priorfun)))
        self.assertEqual(RS.results['priortype'], priortype, msg = str('Expected {}'.format(priortype)))
        self.assertEqual(RS.results['priorpars'], priorpars, msg = str('Expected {}'.format(priorpars)))


#!/usr/bin/env python2
# -*- coding: utf-8 -*-
"""
Created on Wed Jan 17 09:18:19 2018

@author: prmiles
"""

# import required packages
import json
import numpy as np
from ..utilities.NumpyEncoder import NumpyEncoder
from ..utilities.general import removekey
import os

class ResultsStructure:
    '''
    Results from MCMC simulation.

    **Description:** Class used to organize results of MCMC simulation.
    
    Attributes:
        * :meth:`~export_simulation_results_to_json_file`
        * :meth:`~determine_filename`
        * :meth:`~save_json_object`
        * :meth:`~load_json_object`
        * :meth:`~add_basic`
        * :meth:`~add_updatesigma`
        * :meth:`~add_dram`
        * :meth:`~add_prior`
        * :meth:`~add_options`
        * :meth:`~add_model`
        * :meth:`~add_chain`
        * :meth:`~add_s2chain`
        * :meth:`~add_sschain`
        * :meth:`~add_time_stats`
        * :meth:`~add_random_number_sequence`
    '''
    def __init__(self):
        self.results = {} # initialize empty dictionary
        self.basic = False # basic structure not add yet

    # --------------------------------------------------------
    def export_simulation_results_to_json_file(self, results):
        '''
        Export simulation results to a json file.

        Args:
            * **results** (:class:`~.ResultsStructure`): Dictionary of MCMC simulation results/settings.
        '''
        savedir = results['simulation_options']['savedir']
        filename = self.determine_filename(options = results['simulation_options'])
            
        self.save_json_object(results, os.path.join(savedir, filename))
    
    @classmethod
    def determine_filename(cls, options):
        '''
        Determine results filename.
        
        If not specified by `results_filename` in the simulation options, then
        a default naming format is generated using the date string associated
        with the initialization of the simulation.
        
        Args:
            * **options** (:class:`~.SimulationOptions`): MCMC simulation options.
        
        Returns:
            * **filename** (:py:class:`str`): Filename string.
        '''
        results_filename = options['results_filename']
        if results_filename is None:
            dtstr = options['datestr']
            filename = str('{}{}{}'.format(dtstr,'_','mcmc_simulation.json'))
        else:
            filename = results_filename
        return filename
    
    @classmethod
    def save_json_object(cls, results, filename):
        '''
        Save object to json file.

        .. note::

            Filename should include extension.

        Args:
            * **results** (:py:class:`dict`): Object to save.
            * **filename** (:py:class:`str`): Write object into file with this name.
        '''
        with open(filename, 'w') as out:
            json.dump(results, out, sort_keys=True, indent=4, cls=NumpyEncoder)
    
    @classmethod
    def load_json_object(cls, filename):
        '''
        Load object stored in json file.

        .. note::

            Filename should include extension.

        Args:
            * **filename** (:py:class:`str`): Load object from file with this name.

        Returns:
            * **results** (:py:class:`dict`): Object loaded from file.
        '''
        with open(filename, 'r') as obj:
            results = json.load(obj)
        return results
    
    # --------------------------------------------------------
    def add_basic(self, nsimu, covariance, parameters, rejected, simutime, theta):
        '''
        Add basic results from MCMC simulation to structure.

        Args:
            * **nsimu** (:py:class:`int`): Number of MCMC simulations.
            * **model** (:class:`.ModelSettings`): MCMC model settings.
            * **covariance** (:class:`.CovarianceProcedures`): Covariance variables.
            * **parameters** (:class:`.ModelParameters`): Model parameters.
            * **rejected** (:py:class:`dict`): Dictionary of rejection stats.
            * **simutime** (:py:class:`float`): Simulation run time in seconds.
            * **theta** (:class:`~numpy.ndarray`): Last sampled values.
        '''
        
        self.results['theta'] = theta
        
        self.results['parind'] = parameters._parind
        self.results['local'] = parameters._local
        
        self.results['total_rejected'] = rejected['total']*(nsimu**(-1)) # total rejected
        self.results['rejected_outside_bounds'] = rejected['outside_bounds']*(nsimu**(-1)) # rejected due to sampling outside limits
        self.results['R'] = covariance._R
        self.results['qcov'] = np.dot(covariance._R.transpose(),covariance._R)
        self.results['cov'] = covariance._covchain
        self.results['mean'] = covariance._meanchain
        self.results['names'] = [parameters._names[ii] for ii in parameters._parind]
        self.results['limits'] = [parameters._lower_limits[parameters._parind[:]], parameters._upper_limits[parameters._parind[:]]]
             
        self.results['nsimu'] = nsimu
        self.results['simutime'] = simutime
        covariance._qcovorig[np.ix_(parameters._parind,parameters._parind)] = self.results['qcov']
        self.results['qcovorig'] = covariance._qcovorig
        self.basic = True # add_basic has been execute
        
    def add_updatesigma(self, updatesigma, sigma2, S20, N0):
        '''
        Add information to results structure related to observation error.

        Args:
            * **updatesigma** (:py:class:`bool`): Flag to update error variance(s).
            * **sigma2** (:class:`~numpy.ndarray`): Latest estimate of error variance(s).
            * **S20** (:class:`~numpy.ndarray`): Scaling parameter(s).
            * **N0** (:class:`~numpy.ndarray`): Shape parameter(s).

        If :code:`updatesigma is True`, then

        ::

            results['sigma2'] = np.nan
            results['S20'] = S20
            results['N0'] = N0

        Otherwise

        ::

            results['sigma2'] = sigma2
            results['S20'] = np.nan
            results['N0'] = np.nan
        '''
        self.results['updatesigma'] = updatesigma
        if updatesigma:
            self.results['sigma2'] = np.nan
            self.results['S20'] = S20
            self.results['N0'] = N0
        else:
            self.results['sigma2'] = sigma2
            self.results['S20'] = np.nan
            self.results['N0'] = np.nan
    
    def add_dram(self, drscale, RDR, total_rejected, drsettings):
        '''
        Add results specific to performing DR algorithm.

        Args:
            * **drscale** (:class:`~numpy.ndarray`): Reduced scale for sampling in DR algorithm. Default is [5,4,3].
            * **RDR** (:class:`~numpy.ndarray`): Cholesky decomposition of covariance matrix based on DR.
            * **total_rejected** (:py:class:`int`): Number of rejected samples.
            * **drsettings** (:class:`~.DelayedRejection`): Need access to counters within DR class.
        '''
        # extract results from basic structure
        if self.basic is True:
            nsimu = self.results['nsimu']
            
            self.results['drscale'] = drscale
            
            drsettings.iacce[0] = nsimu - total_rejected - sum(drsettings.iacce[1:])
            # 1 - number accepted without DR, 2 - number accepted via DR try 1,
            # 3 - number accepted via DR try 2, etc.
            self.results['iacce'] = drsettings.iacce
            self.results['alpha_count'] = drsettings.dr_step_counter
            self.results['RDR'] = RDR
            return True
        else:
            print('Cannot add DRAM settings to results structure before running ''add_basic''')
            return False
    
    def add_prior(self, mu, sig, priorfun, priortype, priorpars):
        '''
        Add results specific to prior function.

        Args:
            * **mu** (:py:class:`float`): Prior mean.
            * **sig** (:py:class:`float`): Prior standard deviation.
            * **priorfun**: Handle for prior function.
            * **priortype** (:py:class:`int`): Flag identifying type of prior.
            * **priorpars** (:py:class:`float`): Prior parameter for prior update function.

        .. note::

            This feature is not currently implemented.
        '''
        self.results['prior'] = [mu, sig]
        self.results['priorfun'] = priorfun
        self.results['priortype'] = priortype
        self.results['priorpars'] = priorpars
        
    def add_options(self, options = None):
        '''
        Saves subset of features of the simulation options in a nested dictionary.

        Args:
            * **options** (:class:`.SimulationOptions`): MCMC simulation options.
        '''
        # Return options as dictionary
        opt = options.__dict__
        # define list of keywords to NOT add to results structure
        do_not_save_these_keys = ['doram', 'waitbar', 'debug', 'dodram', 'maxmem', 'verbosity', 'RDR', 'stats','initqcovn','drscale','maxiter','_SimulationOptions__options_set', 'skip']
        for keyii in do_not_save_these_keys:
            opt = removekey(opt, keyii)
            
        # must convert 'options' object to a dictionary
        self.results['simulation_options'] = opt

    def add_model(self, model = None):
        '''
        Saves subset of features of the model settings in a nested dictionary.

        Args:
            * **model** (:class:`.ModelSettings`): MCMC model settings.
        '''
        # Return model as dictionary
        mod = model.__dict__
        # define list of keywords to NOT add to results structure
        do_not_save_these_keys = ['sos_function','prior_function','model_function','prior_update_function','prior_pars']
        for keyii in do_not_save_these_keys:
            mod = removekey(mod, keyii)
        # must convert 'model' object to a dictionary
        self.results['model_settings'] = mod
        
    def add_chain(self, chain = None):
        '''
        Add chain to results structure.

        Args:
            * **chain** (:class:`~numpy.ndarray`): Model parameter sampling chain.
        '''
        self.results['chain'] = chain
        
    def add_s2chain(self, s2chain = None):
        '''
        Add observiation error chain to results structure.

        Args:
            * **s2chain** (:class:`~numpy.ndarray`): Sampling chain of observation errors.
        '''
        self.results['s2chain'] = s2chain
        
    def add_sschain(self, sschain = None):
        '''
        Add sum-of-squares chain to results structure.

        Args:
            * **sschain** (:class:`~numpy.ndarray`): Calculated sum-of-squares error for each parameter chains set.
        '''
        self.results['sschain'] = sschain
        
    def add_time_stats(self, mtime, drtime, adtime):
        '''
        Add time spend using each sampling algorithm.

        Args:
            * **mtime** (:py:class:`float`): Time spent performing standard Metropolis.
            * **drtime** (:py:class:`float`): Time spent performing Delayed Rejection.
            * **adtime** (:py:class:`float`): Time spent performing Adaptation.

        .. note::

            This feature is not currently implemented.
        '''
        self.results['time [mh, dr, am]'] = [mtime, drtime, adtime]
        
    def add_random_number_sequence(self, rndseq):
        '''
        Add random number sequence to results structure.

        Args:
            * **rndseq** (:class:`~numpy.ndarray`): Sequence of sampled random numbers.

        .. note::

            This feature is not currently implemented.
        '''
        self.results['rndseq'] = rndseq


#!/usr/bin/env python3
# -*- coding: utf-8 -*-
"""
Created on Wed May 30 05:57:34 2018

@author: prmiles
"""

from pymcmcstat.plotting import utilities
import unittest
from mock import patch
import numpy as np
import math

# --------------------------
class GenerateSubplotGrid(unittest.TestCase):
    def test_generate_subplot_grid(self):
        nparam = 5
        ns1, ns2 = utilities.generate_subplot_grid(nparam = nparam)
        self.assertEqual(ns1, math.ceil(math.sqrt(nparam)), msg = 'Expect 3')
        self.assertEqual(ns2, round(math.sqrt(nparam)), msg = 'Expect 2')

    def test_generate_subplot_grid_1(self):
        nparam = 1
        ns1, ns2 = utilities.generate_subplot_grid(nparam = nparam)
        self.assertEqual(ns1, math.ceil(math.sqrt(nparam)), msg = 'Expect 1')
        self.assertEqual(ns2, round(math.sqrt(nparam)), msg = 'Expect 1')

# --------------------------
class GenerateNames(unittest.TestCase):
    
    def test_default_names(self):
        nparam = 8
        names = utilities.generate_names(nparam = nparam, names = None)
        self.assertEqual(len(names), nparam, msg = 'Length of names should match number of parameters')
        for ii in range(nparam):
            self.assertEqual(names[ii], str('$p_{{{}}}$'.format(ii)))

    def test_names_partial(self):
        nparam = 8
        names = ['hi']
        names = utilities.generate_names(nparam = nparam, names = names)
        self.assertEqual(names[0], 'hi', msg = 'First name is hi')
        for ii in range(1, nparam):
            self.assertEqual(names[ii], str('$p_{{{}}}$'.format(ii)))
 
# --------------------------
class SetupPlotFeatures(unittest.TestCase):
    def test_default_features(self):
        nparam = 2
        ns1, ns2, names, figsizeinches = utilities.setup_plot_features(nparam = nparam, names = None, figsizeinches = None)
        self.assertEqual(ns1, math.ceil(math.sqrt(nparam)), msg = 'Expect 3')
        self.assertEqual(ns2, round(math.sqrt(nparam)), msg = 'Expect 2')
        for ii in range(nparam):
            self.assertEqual(names[ii], str('$p_{{{}}}$'.format(ii)))
        self.assertEqual(figsizeinches, [5,4], msg = 'Default figure size is [5,4]')
        
    def test_nondefault_features(self):
        nparam = 2
        ns1, ns2, names, figsizeinches = utilities.setup_plot_features(nparam = nparam, names = ['hi'], figsizeinches = [7,2])
        self.assertEqual(ns1, math.ceil(math.sqrt(nparam)), msg = 'Expect 3')
        self.assertEqual(ns2, round(math.sqrt(nparam)), msg = 'Expect 2')
        self.assertEqual(names[0], 'hi', msg = 'First name is hi')
        for ii in range(1, nparam):
            self.assertEqual(names[ii], str('$p_{{{}}}$'.format(ii)))
        self.assertEqual(figsizeinches, [7,2], msg = 'Default figure size is [7,2]')
        
# --------------------------
class GenerateDefaultNames(unittest.TestCase):
    
    def test_size_of_default_names(self):
        nparam = 8
        names = utilities.generate_default_names(nparam = nparam)
        self.assertEqual(len(names), nparam, msg = 'Length of names should match number of parameters')
        
    def test_value_of_default_names(self):
        names = utilities.generate_default_names(nparam = 3)
        expected_names = ['$p_{0}$','$p_{1}$','$p_{2}$']
        self.assertEqual(names, expected_names,
                         msg = str('Names do not match: Expected - {}, Received - {}'.format(expected_names, names)))

# --------------------------
class ExtendNamesToMatchNparam(unittest.TestCase):
    
    def test_initially_empty_name_set(self):
        nparam = 3
        names = utilities.extend_names_to_match_nparam(names = None, nparam = nparam)
        expected_names = ['$p_{0}$','$p_{1}$','$p_{2}$']
        self.assertEqual(names, expected_names,
                         msg = str('Names do not match: Expected - {}, Received - {}'.format(expected_names, names)))
        
    def test_single_entry_name_set(self):
        nparam = 3
        names = ['aa']
        names = utilities.extend_names_to_match_nparam(names = names, nparam = nparam)
        expected_names = ['aa','$p_{1}$','$p_{2}$']
        self.assertEqual(names, expected_names,
                         msg = str('Names do not match: Expected - {}, Received - {}'.format(expected_names, names)))

    def test_double_entry_name_set(self):
        nparam = 3
        names = ['aa', 'zz']
        names = utilities.extend_names_to_match_nparam(names = names, nparam = nparam)
        expected_names = ['aa','zz','$p_{2}$']
        self.assertEqual(names, expected_names, 
                         msg = str('Names do not match: Expected - {}, Received - {}'.format(expected_names, names)))
    
# --------------------------
class MakeXGrid(unittest.TestCase):
    
    def test_shape_of_output(self):
        x = np.linspace(0, 10, num = 50)
        npts = 20
        xgrid = utilities.make_x_grid(x = x, npts = 20)
        self.assertEqual(xgrid.shape, (npts, 1), msg = str('Expected return dimension of ({}, 1)'.format(npts)))
        
    def test_default_shape_of_output(self):
        x = np.linspace(0, 10, num = 50)
        xgrid = utilities.make_x_grid(x = x)
        self.assertEqual(xgrid.shape, (100, 1), msg = 'Expected return dimension of (100, 1)')
        
    def test_shape_of_output_for_dense_x(self):
        x = np.linspace(0, 10, num = 500)
        npts = 20
        xgrid = utilities.make_x_grid(x = x, npts = 20)
        self.assertEqual(xgrid.shape, (npts, 1), msg = 'Expected return dimension of (npts, 1)')
        
    def test_default_shape_of_output_for_dense_x(self):
        x = np.linspace(0, 10, num = 500)
        xgrid = utilities.make_x_grid(x = x)
        self.assertEqual(xgrid.shape, (100, 1), msg = 'Expected return dimension of (100, 1)')
    
# --------------------------
class GenerateEllipse(unittest.TestCase):

    def test_does_non_square_matrix_return_error(self):
        cmat = np.zeros([3,2])
        mu = np.zeros([2,1])
        with self.assertRaises(SystemExit):
            utilities.generate_ellipse(mu, cmat)
            
    def test_does_non_symmetric_matrix_return_error(self):
        cmat = np.array([[3,2],[1,3]])
        mu = np.zeros([2,1])
        with self.assertRaises(SystemExit):
            utilities.generate_ellipse(mu, cmat)
            
    def test_does_non_positive_definite_matrix_return_error(self):
        cmat = np.zeros([2,2])
        mu = np.zeros([2,1])
        with self.assertRaises(SystemExit):
            utilities.generate_ellipse(mu, cmat)
            
    def test_does_good_matrix_return_equal_sized_xy_arrays(self):
        cmat = np.eye(2)
        mu = np.zeros([2,1])
        x,y = utilities.generate_ellipse(mu, cmat)
        self.assertEqual(x.shape,y.shape)
        
    def test_does_good_matrix_return_correct_size_array(self):
        cmat = np.eye(2)
        mu = np.zeros([2,1])
        ndp = 50 # number of points to generate ellipse shape
        x,y = utilities.generate_ellipse(mu, cmat, ndp)
        self.assertEqual(x.size, ndp)
        self.assertEqual(y.size, ndp)
        
# --------------------------
class GaussianDensityFunction(unittest.TestCase):
    
    def test_float_return_with_float_input(self):
        self.assertTrue(isinstance(utilities.gaussian_density_function(x = 0.), float),
                             msg = 'Expected float return')
        
    def test_float_return_with_int_input(self):
        self.assertTrue(isinstance(utilities.gaussian_density_function(x = 0), float),
                             msg = 'Expected float return')
        
    def test_float_return_with_float_input_at_nondefault_mean(self):
        self.assertTrue(isinstance(utilities.gaussian_density_function(x = 0., mu = 100), float),
                             msg = 'Expected float return')
        
# --------------------------
class IQrange(unittest.TestCase):
    
    def test_array_return_with_column_vector_input(self):
        x = np.random.random_sample(size = (100,1))
        q = utilities.iqrange(x = x)
        self.assertTrue(isinstance(q, np.ndarray), msg = 'Expected array return - received {}'.format(type(q)))
        self.assertEqual(q.size, 1, msg = 'Expect single element array')
        
    def test_array_return_with_row_vector_input(self):
        x = np.random.random_sample(size = (1,100))
        q = utilities.iqrange(x = x)
        self.assertTrue(isinstance(q, np.ndarray), msg = 'Expected array return - received {}'.format(type(q)))
        self.assertEqual(q.size, 1, msg = 'Expect single element array')
        
# --------------------------
class ScaleBandWidth(unittest.TestCase):
    
    def test_array_return_with_column_vector_input(self):
        x = np.random.random_sample(size = (100,1))
        s = utilities.scale_bandwidth(x = x)
        self.assertTrue(isinstance(s, np.ndarray), msg = 'Expected array return - received {}'.format(type(s)))
        self.assertEqual(s.size, 1, msg = 'Expect single element array')
        
    def test_array_return_with_row_vector_input(self):
        x = np.random.random_sample(size = (1,100))
        s = utilities.scale_bandwidth(x = x)
        self.assertTrue(isinstance(s, np.ndarray), msg = 'Expected array return - received {}'.format(type(s)))
        self.assertEqual(s.size, 1, msg = 'Expect single element array')
        
    @patch('pymcmcstat.plotting.utilities.iqrange', return_value = -1.0)
    def test_array_return_with_iqrange_lt_0(self, mock_iqrange):
        x = np.random.random_sample(size = (1,100))
        s = utilities.scale_bandwidth(x = x)
        self.assertTrue(isinstance(s, np.ndarray), msg = 'Expected array return - received {}'.format(type(s)))
        self.assertEqual(s.size, 1, msg = 'Expect single element array')
        
    @patch('pymcmcstat.plotting.utilities.iqrange', return_value = 1.0)
    def test_array_return_with_iqrange_gt_0(self, mock_iqrange):
        x = np.random.random_sample(size = (1,100))
        s = utilities.scale_bandwidth(x = x)
        self.assertTrue(isinstance(s, np.ndarray), msg = 'Expected array return - received {}'.format(type(s)))
        self.assertEqual(s.size, 1, msg = 'Expect single element array')
        
# --------------------------
class AppendToNrowNcolBasedOnShape(unittest.TestCase):
    def test_shape_is_2d(self):
        nrow = []
        ncol = []
        sh = (2,1)
        nrow, ncol = utilities.append_to_nrow_ncol_based_on_shape(sh = sh, nrow = nrow, ncol = ncol)
        self.assertEqual(nrow, [2], msg = 'Expect [2]')
        self.assertEqual(ncol, [1], msg = 'Expect [1]')
        
    def test_shape_is_1d(self):
        nrow = []
        ncol = []
        sh = (2,)
        nrow, ncol = utilities.append_to_nrow_ncol_based_on_shape(sh = sh, nrow = nrow, ncol = ncol)
        self.assertEqual(nrow, [2], msg = 'Expect [2]')
        self.assertEqual(ncol, [1], msg = 'Expect [1]')
        
# --------------------------
class ConvertFlagToBoolean(unittest.TestCase):
    def test_boolean_conversion(self):
        self.assertTrue(utilities.convert_flag_to_boolean(flag = 'on'), msg = 'on -> True')
        self.assertFalse(utilities.convert_flag_to_boolean(flag = 'off'), msg = 'off -> False')
        
# --------------------------
class SetLocalParameters(unittest.TestCase):
    def test_set_local_parameters(self):
        slp = utilities.set_local_parameters
        self.assertTrue(np.array_equal(slp(ii = 0, local = np.array([0, 0])), np.array([True, True])), msg = 'Expect Array [True, True]')
        self.assertTrue(np.array_equal(slp(ii = 0, local = np.array([0, 1])), np.array([True, False])), msg = 'Expect Array [True, False]')
        self.assertTrue(np.array_equal(slp(ii = 0, local = np.array([1, 0])), np.array([False, True])), msg = 'Expect Array [False, True]')

        self.assertTrue(np.array_equal(slp(ii = 1, local = np.array([0, 1])), np.array([True, True])), msg = 'Expect Array [True, True]')
        self.assertTrue(np.array_equal(slp(ii = 1, local = np.array([1, 0])), np.array([True, True])), msg = 'Expect Array [True, True]')
        
        self.assertTrue(np.array_equal(slp(ii = 1, local = np.array([2, 2])), np.array([False, False])), msg = 'Expect Array [False, False]')
        self.assertTrue(np.array_equal(slp(ii = 2, local = np.array([1, 2])), np.array([False, True])), msg = 'Expect Array [False, True]')
        
# --------------------------------------------
class Empirical_Quantiles_Test(unittest.TestCase):

    def test_does_default_empirical_quantiles_return_3_element_array(self):
        test_out = utilities.empirical_quantiles(np.random.rand(10,1))
        self.assertEqual(test_out.shape, (3,1), msg = 'Default output shape is (3,1)')
        
    def test_does_non_default_empirical_quantiles_return_2_element_array(self):
        test_out = utilities.empirical_quantiles(np.random.rand(10,1), p = np.array([0.2, 0.5]))
        self.assertEqual(test_out.shape, (2,1), msg = 'Non-default output shape should be (2,1)')
        
    def test_empirical_quantiles_should_not_support_list_input(self):
        with self.assertRaises(AttributeError):
            utilities.empirical_quantiles([-1,0,1])
            
    def test_empirical_quantiles_vector(self):
        out = utilities.empirical_quantiles(np.linspace(10,20, num = 10).reshape(10,1), p = np.array([0.22, 0.57345]))
        exact = np.array([[12.2], [15.7345]])
        comp = np.linalg.norm(out - exact)
        self.assertAlmostEqual(comp, 0)
        
# --------------------------------------------
class CheckDefaults(unittest.TestCase):
    def test_check_defaults(self):
        defaults = {'model_display': '-r'}
        kwargs = {'model_display': '--k', 'hi': 3}
        kwargsout = utilities.check_defaults(kwargs, defaults)
        self.assertEqual(kwargsout['model_display'], '--k', msg = 'Expect --k')
        self.assertEqual(kwargsout['hi'], 3, msg = 'Expect 3')
        
    def test_check_defaults_used_defaults(self):
        defaults = {'model_display': '-r'}
        kwargs = {'hi': 3}
        kwargsout = utilities.check_defaults(kwargs, defaults)
        self.assertEqual(kwargsout['model_display'], '-r', msg = 'Expect --k')
        self.assertEqual(kwargsout['hi'], 3, msg = 'Expect 3')


#!/usr/bin/env python2
# -*- coding: utf-8 -*-
"""
Created on Wed Jan 17 09:13:03 2018

@author: prmiles
"""
# import required packages
import numpy as np
import math
import sys
from ..utilities.general import message
# --------------------------
class ModelParameters:
    '''
    MCMC Model Parameters.

    Example:
    ::

        mcstat = MCMC()

        mcstat.parameters.add_model_parameter(name = 'm', theta0 = 1., minimum = -10, maximum = 10)
        mcstat.parameters.add_model_parameter(name = 'b', theta0 = -5., minimum = -10, maximum = 100)
        mcstat.parameters.display_model_parameter_settings()

    This will display to screen:
    ::

        Sampling these parameters:
        name         start [   min,    max] N(  mu, sigma^2)
        m         :   1.00 [-10.00,  10.00] N(0.00, inf)
        b         :  -5.00 [-10.00, 100.00] N(0.00, inf)

    Attributes:
        * :meth:`~add_model_parameter`
        * :meth:`~display_parameter_settings`
    '''
    def __init__(self):
        self.parameters = [] # initialize list
        self.description = 'MCMC model parameters'
    # --------------------------
    def add_model_parameter(self, name = None, theta0 = None, minimum = -np.inf,
                      maximum = np.inf, prior_mu = np.zeros([1]), prior_sigma = np.inf,
                      sample = True, local = 0):
        '''
        Add model parameter to MCMC simulation.

        Args:
            * name (:py:class:`str`): Parameter name
            * theta0 (:py:class:`float`): Initial value
            * minimum (:py:class:`float`): Lower parameter bound
            * maximum (:py:class:`float`): Upper parameter bound
            * prior_mu (:py:class:`float`): Mean value of prior distribution
            * prior_sigma (:py:class:`float`): Standard deviation of prior distribution
            * sample (:py:class:`bool`): Flag to turn sampling on (True) or off (False)
            * local (:py:class:`int`): Local flag - still testing.

        The default prior is a uniform distribution from minimum to maximum parameter value.
        '''
        if name is None:
            name = generate_default_name(len(self.parameters))

        if theta0 is None:
            theta0 = 1.0

        # append dictionary element
        self.parameters.append({'name': name, 'theta0': theta0, 'minimum': minimum,
                                'maximum': maximum, 'prior_mu': prior_mu, 'prior_sigma': prior_sigma,
                                'sample': sample, 'local': local})
    # --------------------------
    def _openparameterstructure(self, nbatch):
        
        # unpack input object
        parameters = self.parameters
        npar = len(parameters)
        
        # initialize arrays - as lists and numpy arrays (improved functionality)
        self._names = []
        self._initial_value = np.zeros(npar)
        self._value = np.zeros(npar)
        self._parind = np.ones(npar, dtype = int)
        self._local = np.zeros(npar)
        self._upper_limits = np.ones(npar)*np.inf
        self._lower_limits = -np.ones(npar)*np.inf
        self._thetamu = np.zeros(npar)
        self._thetasigma = np.ones(npar)*np.inf
                
        # scan for local variables
        ii = 0
        for kk in range(npar):
            if parameters[kk]['sample'] == 0:
                if parameters[kk]['local'] != 0:
                    self._local[ii:(ii+nbatch-1)] = range(0,nbatch)
                    npar = npar + nbatch - 1
                    ii = ii + nbatch - 1

            ii += 1 # update counter
            
        ii = 0
        for kk in range(npar):
            if self._local[ii] == 0:
                self._names.append(parameters[kk]['name'])
                self._initial_value[ii] = parameters[kk]['theta0']
                self._value[ii] = parameters[kk]['theta0']
                # default values defined in "Parameters" class in classes.py
                # define lower limits
                self._lower_limits[ii] = parameters[kk]['minimum']
                # define upper limits
                self._upper_limits[ii] = parameters[kk]['maximum']
                # define prior mean
                self._thetamu[ii] = parameters[kk]['prior_mu']
                if np.isnan(self._thetamu[ii]):
                    self._thetamu[ii] = self.value[ii]
                # define prior standard deviation
                self._thetasigma[ii] = parameters[kk]['prior_sigma']
                if self._thetasigma[ii] == 0:
                    self._thetasigma[ii] = np.inf
                # turn sampling on/off
                self._parind[ii] = parameters[kk]['sample']
            ii += 1 # update counter
            
        # make parind list of nonzero elements
        self._parind = np.flatnonzero(self._parind)
        
        self.npar = len(self._parind) # append number of parameters to structure
    # --------------------------            
    def _results_to_params(self, results, use_local = 1):
    
        # unpack results dictionary
        parind = results['parind']
        names = results['names']
        local = results['local']
        theta = results['theta']

        for ii, parii in enumerate(parind):
            if use_local == 1 and local[parii] == 1:
                name = names[ii] # unclear usage
            else:
                name = names[ii]
    
            for kk in range(len(self.parameters)):
                if name == self.parameters[kk]['name']:
                    # change NaN prior mu (=use initial) to the original initial value
                    if np.isnan(self.parameters[kk]['prior_mu']):
                        self.parameters[kk]['prior_mu'] = self.parameters[kk]['theta0']
                        
                    # only change if parind = 1 in params (1 is the default)
                    if self.parameters[kk]['sample'] == 1 or self.parameters[kk]['sample'] is None:
                        self.parameters[kk]['theta0'] = theta[parii]
                        
    # --------------------------
    def _check_initial_values_wrt_parameter_limits(self):
        # check initial parameter values are inside range
        if (self._initial_value[np.ix_(self._parind)] < self._lower_limits[np.ix_(self._parind)]).any() or (self._initial_value[np.ix_(self._parind)] > self._upper_limits[np.ix_(self._parind)]).any():
            # proposed value outside parameter limits
            sys.exit('Proposed value outside parameter limits - select new initial parameter values')
        else:
             return True
    # --------------------------     
    def _check_prior_sigma(self, verbosity):
        message(verbosity, 2, 'If prior variance <= 0, setting to Inf\n')
        self._thetasigma = replace_list_elements(self._thetasigma, less_than_or_equal_to_zero, float('Inf'))
    # --------------------------
    def display_parameter_settings(self, verbosity = None, noadaptind = None):
        '''
        Display parameter settings

        Args:
            * **verbosity** (:py:class:`int`): Verbosity of display output. :code:`0`
            * **noadaptind** (:py:class:`int`): Indices not to be adapted in covariance matrix. :code:`[]`
        '''
        parind = self._parind
        names = self._names
        value = self._initial_value
        lower_limits = self._lower_limits
        upper_limits = self._upper_limits
        theta_mu = self._thetamu
        theta_sigma = self._thetasigma
        
        verbosity = check_verbosity(verbosity)
        noadaptind = check_noadaptind(noadaptind)
            
        if noadaptind is None:
            noadaptind = []
        
        if verbosity > 0:
            print('\nSampling these parameters:')
            print('{:>10s} {:>10s} [{:>9s}, {:>9s}] N({:>9s}, {:>9s})'.format('name',
                  'start', 'min', 'max', 'mu', 'sigma^2'))
            nprint = len(parind)
            for ii in range(nprint):
                name = str('{:>10s}'.format(names[parind[ii]]))
                valuestr = format_number_to_str(value[parind[ii]])
                lowstr = format_number_to_str(lower_limits[parind[ii]])
                uppstr = format_number_to_str(upper_limits[parind[ii]])
                mustr = format_number_to_str(theta_mu[parind[ii]])
                sigstr = format_number_to_str(theta_sigma[parind[ii]])
                
                st = noadapt_display_setting(ii, noadaptind)
                h2 = prior_display_setting(x = theta_sigma[parind[ii]])
                
                print('{:s}: {:s} [{:s}, {:s}] N({:s},{:s}{:s}){:s}'.format(name, valuestr, lowstr, uppstr, mustr, sigstr, h2, st))

# --------------------------
def replace_list_elements(x, testfunction, value):
    '''
    Replace list elements based on results from testfunction.

    Args:
        * **x** (:py:class:`list`): List of numbers to be tested
        * **testfunction** (:py:func:`testfunction`): Test function
        * **value** (:py:class:`float`): Value to assign if test function return True

    Returns:
        * **x** (:py:class:`list`): Updated list
    '''
    for ii, xii in enumerate(x):
        if testfunction(xii):
            x[ii] = value
    return x
# --------------------------
def generate_default_name(nparam):
    '''
    Generate generic parameter name.
    For example, if :code:`nparam = 4`, then the generated name is::

        names = 'p_{3}'

    Args:
        * **nparam** (:py:class:`int`): Number of parameter names to generate

    Returns:
        * **name** (:py:class:`str`): Name based on size of parameter list
    '''
    return (str('$p_{{{}}}$'.format(nparam)))
# --------------------------
def check_verbosity(verbosity):
    '''
    Check if verbosity is None -> 0

    Args:
        * **verbosity** (:py:class:`int`): Verbosity level

    Returns:
        * **verbosity** (:py:class:`int`): Returns 0 if verbosity was initially `None`
    '''
    if verbosity is None:
        verbosity = 0
    return verbosity

# --------------------------
def check_noadaptind(noadaptind):
    '''
    Check if noadaptind is None -> Empty List

    Args:
        * **noadaptind** (:py:class:`list`): Indices not to be adapted in covariance matrix.

    Returns:
        * **noadaptind** (:py:class:`list`): Indices not to be adapted in covariance matrix.
    '''
    if noadaptind is None:
        noadaptind = []
    return noadaptind
# --------------------------
def noadapt_display_setting(ii, noadaptind):
    '''
    Define display settins if index not being adapted.

    Args:
        * **ii** (:py:class:`int`): Current index number
        * **noadaptind** (:py:class:`list`): List of indices not being adapted.

    Returns:
        * **st** (:py:class:`str`): String to be displayed, depending on if in `noadaptind`.
    '''
    if ii in noadaptind: # THIS PARAMETER IS FIXED
        st = ' (*)'
    else:
        st = ''
    return st
# --------------------------
def prior_display_setting(x):
    '''
    Define display string for prior.

    Args:
        * **x** (:py:class:`float`): Prior mean

    Returns:
        * **h2** (:py:class:`str`): String to be displayed, depending on if `x` is infinity.
    '''
    if math.isinf(x):
        h2 = ''
    else:
        h2 = '^2'
    return h2
# --------------------------    
def format_number_to_str(number):
    '''
    Format number for display

    Args:
        * **number** (:py:class:`float`): Number to be formatted

    Returns:
        * (:py:class:`str`): Formatted string display
    '''
    if abs(number) >= 1e4 or abs(number) <= 1e-2:
        return str('{:9.2e}'.format(number))
    else:
        return str('{:9.2f}'.format(number))
# --------------------------
def less_than_or_equal_to_zero(x):
    '''
    Return result of test on number based on less than or equal to

    Args:
        * **x** (:py:class:`float`): Number to be tested

    Returns:
        * (:py:class:`bool`): Result of test: `x<=0`
    '''
    return (x<=0)


#!/usr/bin/env python3
# -*- coding: utf-8 -*-
"""
Created on Wed Jun 27 12:07:11 2018

Utility functions used by different samplers

@author: prmiles
"""

import numpy as np

# --------------------------------------------------------
def sample_candidate_from_gaussian_proposal(npar, oldpar, R):
    '''
    Sample candidate from Gaussian proposal distribution

    Args:
        * **npar** (:py:class:`int`): Number of parameters being samples
        * **oldpar** (:class:`~numpy.ndarray`): :math:`q^{k-1}` Old parameter set.
        * **R** (:class:`~numpy.ndarray`): Cholesky decomposition of parameter covariance matrix.

    Returns:
        * **newpar** (:class:`~numpy.ndarray`): :math:`q^*` - candidate
        * **npar_sample_from_sample** (:class:`~numpy.ndarray`): Sampled values from normal distibution: :math:`N(0,1)`.
    '''
    npar_sample_from_normal = np.random.randn(1, npar)
    newpar = oldpar + np.dot(npar_sample_from_normal, R)
    newpar = newpar.reshape(npar)
    return newpar, npar_sample_from_normal

# --------------------------------------------------------
def is_sample_outside_bounds(theta, lower_limits, upper_limits):
    '''
    Check whether proposal value is outside parameter limits

    Args:
        * **theta** (:class:`~numpy.ndarray`): Value of sampled model parameters
        * **lower_limits** (:class:`~numpy.ndarray`): Lower limits
        * **upper_limits** (:class:`~numpy.ndarray`): Upper limits

    Returns:
        * **outsidebounds** (:py:class:`bool`): True -> Outside of parameter limits
    '''
    if (theta < lower_limits).any() or (theta > upper_limits).any():
        outsidebounds = True
    else:
        outsidebounds = False
    return outsidebounds

# --------------------------------------------------------
def acceptance_test(alpha):
    '''
    Run standard acceptance test

    .. math::

        & \\text{If}~u_{\\alpha} <~\\alpha, \\

        & \\quad \\text{Set}~q^k = q^*,~SS_{q^k} = SS_{q^*} \\

        & \\text{Else} \\

        & \\quad \\text{Set}~q^k = q^{k-1},~SS_{q^k} = SS_{q^{k-1}}

    Args:
        * **alpha** (:py:class:`float`): Result of likelihood function

    Returns:
        * **accept** (:py:class:`bool`): False - reject, True - accept
    '''
    if alpha >= 1 or alpha > np.random.rand(1,1):
        accept = True
    else:
        accept = False
    return accept

# -------------------------------------------
def set_outside_bounds(next_set):
    '''
    Assign set features based on being outside bounds

    Args:
        * **next_set** (:class:`~.ParameterSet`): :math:`q^*`

    Returns:
        * **next_set** (:class:`~.ParameterSet`): :math:`q^*` with updated features
        * **outbound** (:class:`bool`): True
    '''
    next_set.alpha = 0
    next_set.prior = 0
    next_set.ss = np.inf
    outbound = True
    
    return next_set, outbound


# -*- coding: utf-8 -*-

"""
Created on Mon Jan. 15, 2018

Description: This file contains a series of utilities designed to test the
features in the 'PriorFunction.py" class of the pymcmcstat module.

@author: prmiles
"""
from pymcmcstat.procedures.PriorFunction import PriorFunction
import unittest
import test.general_functions as gf

# --------------------------
# PriorFunction
# --------------------------
class Initialize_Prior_Function(unittest.TestCase):

    def test_PF_default_mu_sigma(self):
        PF = PriorFunction()
        PFD = PF.__dict__
        PFD = gf.removekey(PFD, 'priorfun')
        defaults = {'mu': None, 'sigma': None}
        for (k,v) in PFD.items():
            self.assertEqual(v, defaults[k], msg = str('Default {} is {}'.format(k, defaults[k])))

    def test_PS_defaul_priorfun(self):
        key = 'priorfun'
        PF = PriorFunction()
        PFD = PF.__dict__
        self.assertEqual(PFD[key], PF.default_priorfun, msg = str('Expected {} = {}'.format(key, 'PriorFunction.default_priorfun')))
        
class Evaluation_Default_Prior_Function(unittest.TestCase):
    
    def test_PF_evaluation_with_single_set(self):
        PF = PriorFunction()
        theta = [0.]
        mu = [0.]
        sigma = [0.1]
        self.assertEqual(PF.default_priorfun(theta = theta, mu = mu, sigma=sigma), 0, msg = 'Prior should be 0')
        
    def test_PF_evaluation_with_double_set(self):
        PF = PriorFunction()
        theta = [0., 0.]
        mu = [0., 0.]
        sigma = [0.1, 0.1]
        self.assertEqual(PF.default_priorfun(theta = theta, mu = mu, sigma=sigma), 0, msg = 'Prior should be 0')
        
    def test_PF_evaluation_with_single_set_and_nonzero_answer(self):
        PF = PriorFunction()
        theta = [1.]
        mu = [45.]
        sigma = [0.1]
        self.assertTrue(PF.default_priorfun(theta = theta, mu = mu, sigma=sigma) > 0, msg = 'Prior should be >=0')
        
    def test_PF_evaluation_with_double_set_and_nonzero_answer(self):
        PF = PriorFunction()
        theta = [1., 91.]
        mu = [45., 27.]
        sigma = [0.1, 16.]
        self.assertTrue(PF.default_priorfun(theta = theta, mu = mu, sigma=sigma) > 0, msg = 'Prior should be >=0')
        
class EvaluatePriorFunction(unittest.TestCase):
    
    def test_EPF_with_single_set(self):
        PF = PriorFunction()
        theta = [0.]
        self.assertEqual(PF.evaluate_prior(theta = theta), 0, msg = 'Prior should be 0')
        
    def test_EPF_with_double_set(self):
        PF = PriorFunction()
        theta = [0., 0.]
        self.assertEqual(PF.evaluate_prior(theta = theta), 0, msg = 'Prior should be 0')
        
    def test_EPF_with_single_set_and_nonzero_answer(self):
        PF = PriorFunction()
        theta = [1.]
        self.assertTrue(PF.evaluate_prior(theta = theta) > 0, msg = 'Prior should be >=0')


#!/usr/bin/env python3
# -*- coding: utf-8 -*-
"""
Created on Tue May  1 15:58:22 2018

@author: prmiles
"""

from .MCMC import MCMC
from .chain import ChainStatistics as CS
from .samplers.utilities import is_sample_outside_bounds
from multiprocessing import Pool, cpu_count
import numpy as np
import sys
import os
import copy
import time

class ParallelMCMC:
    '''
    Run Parallel MCMC Simulations.

    Attributes:
        * :meth:`~setup_parallel_simulation`
        * :meth:`~run_parallel_simulation`
        * :meth:`~display_individual_chain_statistics`
    '''
    def __init__(self):
        self.description = 'Run MCMC simulations in parallel'

    def setup_parallel_simulation(self, mcset, initial_values = None, num_cores = 1, num_chain = 1):
        '''
        Setup simulation to run in parallel.
        
        Settings defined in `mcset` object will be copied into different instances in
        order to run parallel chains.
        
        Args:
            * **mcset** (:class:`~.MCMC`): Instance of MCMC object with serial simulation setup.
            * **initial_values** (:class:`~numpy.ndarray`): Array of initial parameter values - [num_chain,npar].
            * **num_cores** (:py:class:`int`): Number of cores designated by user.
            * **num_chain** (:py:class:`int`): Number of sampling chains to be generated.
        '''
        # extract settings from mcset
        data, options, model, parameters = unpack_mcmc_set(mcset = mcset)
        npar, low_lim, upp_lim = get_parameter_features(parameters.parameters)
        options = check_options_output(options)
        
        # number of CPUs
        self.num_cores = assign_number_of_cores(num_cores)
        
        # check initial values and number of chains to be generated
        self.initial_values, self.num_chain = check_initial_values(initial_values = initial_values, num_chain = num_chain, npar = npar, low_lim = low_lim, upp_lim = upp_lim)
        
        # assign parallel log file directory
        parallel_dir = options.savedir
        check_directory(parallel_dir)
        self.__parallel_dir = parallel_dir
            
        # replicate features of mcstat object num_chain times
        self.parmc = []
        for ii in range(self.num_chain):
            
            self.parmc.append(MCMC())
            # replicate data
            self.parmc[ii].data = copy.deepcopy(data)
            # replicate model settings
            self.parmc[ii].model_settings = copy.deepcopy(model)
            # replicate simulation options and create log files for each
            self.parmc[ii].simulation_options = copy.deepcopy(options)
            chain_dir = str('chain_{}'.format(ii))
            self.parmc[ii].simulation_options.savedir = str('{}{}{}'.format(self.__parallel_dir,os.sep,chain_dir))
            # replicate parameter settings and assign distributed initial values
            self.parmc[ii].parameters = copy.deepcopy(parameters)
            for jj in range(npar):
                self.parmc[ii].parameters.parameters[jj]['theta0'] = self.initial_values[ii,jj]
            
    def run_parallel_simulation(self):
        '''
        Run MCMC simulations in parallel.
        
        The code is run in parallel by using :class:`~.Pool`.  While
        running, you can expect a display similar to

        ::
            
            Processing: <parallel_dir>/chain_1
            Processing: <parallel_dir>/chain_0
            Processing: <parallel_dir>/chain_2
            
        The simulation is complete when you see the run time displayed.
        
        ::
            
            Parallel simulation run time: 16.15234899520874 sec
        '''
        start = time.time()
        mcpool = Pool(processes=self.num_cores) # depends on available cores
        res = mcpool.map(run_serial_simulation, self.parmc)
        mcpool.close() # not optimal! but easy
        mcpool.join()
        end = time.time()
        self.__parsimutime = end - start
        print('Parallel simulation run time: {} sec'.format(self.__parsimutime))
        # assign results to invidual simulations
        for ii in range(self.num_chain):
            self.parmc[ii].simulation_results = res[ii]

    # -------------------------
    def display_individual_chain_statistics(self):
        '''
        Display chain statistics for different chains in parallel simulation.
        
        Example display:

        ::
            
            ****************************************
            Displaying results for chain 0
            Files: <parallel_dir>/chain_0
            
            ---------------------
            name      :       mean        std     MC_err        tau     geweke
            m         :     1.9869     0.1295     0.0107   320.0997     0.9259
            b         :     3.0076     0.2489     0.0132   138.1260     0.9413
            ---------------------
            
            ****************************************
            Displaying results for chain 1
            Files: <parallel_dir>/chain_1
            
            ---------------------
            name      :       mean        std     MC_err        tau     geweke
            m         :     1.8945     0.4324     0.0982  2002.6361     0.3116
            b         :     3.2240     1.0484     0.2166  1734.0201     0.4161
            ---------------------
        '''
        for ii, mc in enumerate(self.parmc):
            print('\n{}\nDisplaying results for chain {}\nFiles: {}'.format(40*'*',ii,mc.simulation_options.savedir))
            CS.chainstats(mc.simulation_results.results['chain'],mc.simulation_results.results)

# -------------------------
def unpack_mcmc_set(mcset):
    '''
    Unpack attributes of MCMC object.
    
    Args:
        * **mcset** (:class:`~.MCMC`): MCMC object.
        
    Returns:
        * **data** (:class:`~.DataStructure`): MCMC data structure.
        * **options** (:class:`~.SimulationOptions`): MCMC simulation options.
        * **model** (:class:`~.ModelSettings`): MCMC model settings.
        * **parameters** (:class:`~.ModelParameters`): MCMC model parameters.
    '''
    data = mcset.data
    options = mcset.simulation_options
    model = mcset.model_settings
    parameters = mcset.parameters
    return data, options, model, parameters
# -------------------------
def get_parameter_features(parameters):
    '''
    Get features of model parameters.
    
    Args:
        * **parameters** (:py:class:`list`): List of MCMC model parameter dictionaries.

    Returns:
        * **npar** (:py:class:`int`): Number of model parameters.
        * **low_lim** (:class:`~numpy.ndarray`): Lower limits.
        * **upp_lim** (:class:`~numpy.ndarray`): Upper limits.
    '''
    npar = len(parameters)
    theta0 = np.zeros([npar])
    low_lim = np.zeros([npar])
    upp_lim = np.zeros([npar])
    for jj in range(npar):
        theta0[jj] = parameters[jj]['theta0']
        low_lim[jj] = parameters[jj]['minimum']
        upp_lim[jj] = parameters[jj]['maximum']
        # check if infinity - needs to be finite for default mapping
        if low_lim[jj] == -np.inf:
            low_lim[jj] = theta0[jj] - 100*(np.abs(theta0[jj]))
            print('Finite lower limit required - setting low_lim[{}] = {}'.format(jj,low_lim[jj]))
        if upp_lim[jj] == np.inf:
            upp_lim[jj] = theta0[jj] + 100*(np.abs(theta0[jj]))
            print('Finite upper limit required - setting upp_lim[{}] = {}'.format(jj,upp_lim[jj]))
    return npar, low_lim, upp_lim
# -------------------------
def check_initial_values(initial_values, num_chain, npar, low_lim, upp_lim):
    '''
    Check if initial values satisfy requirements.
    
    Args:
        * **initial_values** (:class:`~numpy.ndarray`): Array of initial parameter values - [num_chain,npar].
        * **num_chain** (:py:class:`int`): Number of sampling chains to be generated.
        * **npar** (:py:class:`int`): Number of model parameters.
        * **low_lim** (:class:`~numpy.ndarray`): Lower limits.
        * **upp_lim** (:class:`~numpy.ndarray`): Upper limits.

    Returns:
        * **initial_values** (:class:`~numpy.ndarray`): Array of initial parameter values - [num_chain,npar].
        * **num_chain** (:py:class:`int`): Number of sampling chains to be generated.
    '''
    if initial_values is None:
        initial_values = generate_initial_values(num_chain = num_chain, npar = npar, low_lim = low_lim, upp_lim = upp_lim)
    else:
        num_chain, initial_values = check_shape_of_users_initial_values(initial_values = initial_values, num_chain = num_chain, npar = npar)
        initial_values = check_users_initial_values_wrt_limits(initial_values = initial_values, low_lim = low_lim, upp_lim = upp_lim)
    return initial_values, num_chain
# -------------------------
def generate_initial_values(num_chain, npar, low_lim, upp_lim):
    '''
    Generate initial values by sampling from uniform distribution between limits
    
    Args:
        * **num_chain** (:py:class:`int`): Number of sampling chains to be generated.
        * **npar** (:py:class:`int`): Number of model parameters.
        * **low_lim** (:class:`~numpy.ndarray`): Lower limits.
        * **upp_lim** (:class:`~numpy.ndarray`): Upper limits.

    Returns:
        * **initial_values** (:class:`~numpy.ndarray`): Array of initial parameter values - [num_chain,npar]
    '''
    u = np.random.random_sample(size = (num_chain, npar))
    # map sampling between lower/upper limit
    initial_values = low_lim + (upp_lim - low_lim)*u
    return initial_values
# -------------------------
def check_shape_of_users_initial_values(initial_values, num_chain, npar):
    '''
    Check shape of users initial values
    
    Args:
        * **initial_values** (:class:`~numpy.ndarray`): Array of initial parameter values - expect [num_chain,npar]
        * **num_chain** (:py:class:`int`): Number of sampling chains to be generated.
        * **npar** (:py:class:`int`): Number of model parameters.

    Returns:
        * **num_chain** (:py:class:`int`): Number of sampling chains to be generated - equal to number of rows in initial values array.
        * **initial_values**
    '''
    m,n = initial_values.shape
    if m != num_chain:
        print('Shape of initial values inconsistent with requested number of chains.  \n num_chain = {}, initial_values.shape -> {},{}.  Resizing num_chain to {}.'.format(num_chain, m, n, m))
        num_chain = m
    if n != npar:
        print('Shape of initial values inconsistent with requested number of parameters.  \n npar = {}, initial_values.shape -> {},{}.  Only using first {} columns of initial_values.'.format(npar, m, n, npar))
        initial_values = np.delete(initial_values, [range(npar,n+1)], axis = 1)
    return num_chain, initial_values
# -------------------------
def check_users_initial_values_wrt_limits(initial_values, low_lim, upp_lim):
    '''
    Check users initial values wrt parameter limits
    
    Args:
        * **initial_values** (:class:`~numpy.ndarray`): Array of initial parameter values - expect [num_chain,npar]
        * **low_lim** (:class:`~numpy.ndarray`): Lower limits.
        * **upp_lim** (:class:`~numpy.ndarray`): Upper limits.

    Returns:
        * **initial_values** (:class:`~numpy.ndarray`): Array of initial parameter values - expect [num_chain,npar]

    Raises:
        * `SystemExit` if initial values are outside parameter bounds.
    '''
    outsidebounds = is_sample_outside_bounds(initial_values, low_lim, upp_lim)
    if outsidebounds is True:
        sys.exit(str('Initial values are not within parameter limits.  Make sure they are within the following limits:\n\tLower: {}\n\tUpper: {}\nThe initial_values tested were:\n{}'.format(low_lim, upp_lim, initial_values)))
    else:
        return initial_values
# -------------------------
def check_options_output(options):
    '''
    Check output settings defined in options
    
    Args:
        * **options** (:class:`.SimulationOptions`): MCMC simulation options.

    Returns:
        * **options** (:class:`.SimulationOptions`): MCMC simulation options with at least binary save flag set to True.
    '''
    if options.save_to_txt == False and options.save_to_bin == False:
        options.save_to_bin = True
    return options

# -------------------------
def check_directory(directory):
    '''
    Check and make sure directory exists
    
    Args:
        * **directory** (:py:class:`str`): Folder/directory path name.
    '''
    if not os.path.exists(directory):
        os.makedirs(directory)
        
# -------------------------
def run_serial_simulation(mcstat):
    '''
    Run serial MCMC simulation
    
    Args:
        * **mcstat** (:class:`MCMC.MCMC`): MCMC object.

    Returns:
        * **results** (:py:class:`dict`): Results dictionary for serial simulation.
    '''
    print('Processing: {}'.format(mcstat.simulation_options.savedir))
    mcstat.run_simulation()
    return mcstat.simulation_results

# -------------------------
def assign_number_of_cores(num_cores = 1):
    '''
    Assign number of cores to use in parallel process
    
    Args:
        * **num_cores** (:py:class:`int`): Number of cores designated by user.

    Returns:
        * **num_cores** (:py:class:`int`): Number of cores designated by user or maximum number of cores available on machine.
    '''
    tmp = cpu_count()
    if num_cores > tmp:
        return tmp
    else:
        return num_cores